local frame, events = CreateFrame("Frame"), {};
function events:PLAYER_ENTERING_WORLD(...) -- this is called every time a player zones into a new area
		-- Find out what type of area we just entered
	inInstance, instanceType = IsInInstance();
		
	if (instanceType ~= "none") then
		-- we're in an instance so now we check durability
		durabilityStatus = getDurabilityStatus();
		if (durabilityStatus == "broken") then
			print("AMFG_LazyDurability: Broken.");
			StaticPopup_Show("BROKEN_DURABILITY")
		elseif (durabilityStatus == "low") then
			print("AMFG_LazyDurability: low.");
			StaticPopup_Show("LOW_DURABILITY")
		end
	end
	
end
function events:ADDON_LOADED(...)
 -- Initialize our popup dialog which appears if durability is low
	StaticPopupDialogs["LOW_DURABILITY"] = {
	  text = "Durability low!",
	  button1 = "OK",
	  timeout = 0,
	  whileDead = false,
	  hideOnEscape = true,
	  showAlert = true
	}
	
	StaticPopupDialogs["BROKEN_DURABILITY"] = {
	  text = "One or more items at 0 durability!",
	  button1 = "OK",
	  timeout = 0,
	  whileDead = false,
	  hideOnEscape = true,
	  showAlert = true
	}	
end
frame:SetScript("OnEvent", function(self, event, ...)
 events[event](self, ...); -- call one of the functions above
end);
for k, v in pairs(events) do
 frame:RegisterEvent(k); -- Register all events for which handlers have been defined
end



function getDurabilityStatus()

	local itemList = 
	{
		[1] = "HeadSlot",
		[2] = "NeckSlot",
		[3] = "ShoulderSlot",
		[4] = "BackSlot",
		[5] = "ChestSlot",
		[6] = "WristSlot",
		[7] = "HandsSlot",
		[8] = "WaistSlot",
		[9] = "LegsSlot",
		[10] = "FeetSlot",
		[11] = "MainHandSlot",
		[12] = "SecondaryHandSlot",
		[13] = "RangedSlot"
	};
	
	for i=1,13 do 
		-- Check to see if anything is broken, if so, return "broken"
		slotId, textureName = GetInventorySlotInfo(itemList[i]);
		current, maximum = GetInventoryItemDurability(slotId);
		
		if (current == 0) then
			return "broken";
		end		
	end
	
	for j=1,13 do 
		-- Check to see if anything is low durability (25% or lower), if so return "low"
		slotId, textureName = GetInventorySlotInfo(itemList[j]);
		current, maximum = GetInventoryItemDurability(slotId);
				
		if (current ~= nil) then
			if ((current/maximum) <= .25) then
				return "low";
			end
		end		
	end
		-- If durability is fine return whatever.
		return 0;
end
	