--[[
    Title: DKM
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")
DKM = LibStub("AceAddon-3.0"):NewAddon("DKM", "AceConsole-3.0", "AceEvent-3.0")

local Sticky = LibStub("LibStickyFrames-2.0")

DKM_UIFont = CreateFont("DKM_UIFont14")
DKM_UIFont:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
DKM_UIFont:SetJustifyV("MIDDLE")
DKM_UIFont:SetJustifyH("CENTER")

DKM_UIFont = CreateFont("DKM_UIFont16")
DKM_UIFont:SetFont("Fonts\\FRIZQT__.TTF", 16, "OUTLINE")
DKM_UIFont:SetJustifyV("MIDDLE")
DKM_UIFont:SetJustifyH("CENTER")

DKM_UIFont = CreateFont("DKM_UIFont18")
DKM_UIFont:SetFont("Fonts\\FRIZQT__.TTF", 18, "OUTLINE")
DKM_UIFont:SetJustifyV("MIDDLE")
DKM_UIFont:SetJustifyH("CENTER")

DKM.version = {
    major = 2,
    minor = 0,
    revision = 1
}

DKM.gfxPacksRunesTextures = {
    ["Blizzard"] = "Blizzard"
}

DKM.gfxPacksRunesBorders = {
    ["Blizzard"] = "Blizzard"
}

DKM.gfxPacksTrackersTextures = {
    ["Blizzard"] = "Blizzard"
}

DKM.gfxPacksTrackersBorders = {
    ["None"] = "None"
}

DKM.gfxPackRunesTexturesData = {
    ["Blizzard"] = {
        path = "Interface\\PlayerFrame",
        form = "square"
    }
}

DKM.gfxPackRunesBordersData = {
    ["Blizzard"] = {
        path = "Interface\\PlayerFrame",
        form = "square"
    }
}

DKM.gfxPackTrackersTexturesData = {
    ["Blizzard"] = {
        path = "Interface\\ICONS",
        form = "square"
    }
}

DKM.gfxPackTrackersBordersData = {
    ["None"] = {
        path = "Interface\\ICONS",
        form = "square"
    }
}

DKM.textureFormCooldownScaleFactor = {
    ["square"] = 1.0,
	["circle"] = 0.71
}

local options = {
    name = "DKM",
    handler = DKM,
    type = 'group',
    args = {
        general = {
            name = "General",
            type = "group",
            desc = "General options",
            args = {
                lockWidgets = {
                    type = "toggle",
                    width = "full",
                    name = L["Lock all widgets"],
                    desc = L["All widgets are visible when unlocked. Also when you drag them, they stick to each other. To drag without sticking hold SHIFT."],
                    get = "IsLockWidgets",
                    set = "ToggleLockWidgets",
                },
                uiScale = {
                    type = "range",
                    name = L["Global UI Scale"],
                    desc = L["Scales the entire DKM UI to a factor of the default."],
                    min = 0.1,
                    max = 2,
                    step = 0.05,
                    get = "GetUIScale",
                    set = "SetUIScale",
                },
                uiOpacityInCombat = {
                    type = "range",
                    name = L["UI opacity in combat"],
                    desc = L["Sets in combat opacity for the entire DKM UI."],
                    min = 0.05,
                    max = 1,
                    step = 0.05,
                    get = "GetUIOpacityInCombat",
                    set = "SetUIOpacityInCombat",
                },
                uiOpacityOutOfCombat = {
                    type = "range",
                    name = L["UI opacity out of combat"],
                    desc = L["Sets out of combat opacity for the entire DKM UI."],
                    min = 0,
                    max = 1,
                    step = 0.05,
                    get = "GetUIOpacityOutOfCombat",
                    set = "SetUIOpacityOutOfCombat",
                },
            },
        },
    },
}

local defaults = {
    profile = {
        general = {
            lockWidgets = true,
            uiScale = 1.0,
            uiOpacityInCombat = 1.0,
            uiOpacityOutOfCombat = 1.0,
        },
        trackers = {
            enabled = true,
            texture_pack = "Blizzard",
			border_pack = "None",
            uiScale = 1.0,
            showCooldown = true,
            showText = true,
            subSecondPrescision = true,
            subSecondPrescisionThreshold = 3.5,
            showEbonPlague = true,
            showScarletFever = false,
			showUnholyBlight = false,
			showShadowInfusion = true,
			showDarkTransformation = true,
			showBladeBarrier = true,
			showNecroticStrike = true,
			showSuddenDoom = true,
			showKillingMachine = true,
			showRime = true,
			showCrimsonScourge = true,
            fallOffPulse = 5,
			trackable = {
				[1] = {
					relativePoint = "CENTER",
					coord_x = 65,
					coord_y = -160
				},
				[2] = {
					relativePoint = "CENTER",
					coord_x = -65,
					coord_y = -160
				},
				[3] = {
					relativePoint = "CENTER",
					coord_x = 0,
					coord_y = -200
				},
				[4] = {
					relativePoint = "CENTER",
					coord_x = 50,
					coord_y = -200
				},
				[5] = {
					relativePoint = "CENTER",
					coord_x = 200,
					coord_y = 0
				},
				[6] = {
					relativePoint = "CENTER",
					coord_x = 250,
					coord_y = 0
				},
				[7] = {
					relativePoint = "CENTER",
					coord_x = 300,
					coord_y = 0
				},
				[8] = {
					relativePoint = "CENTER",
					coord_x = 350,
					coord_y = 0
				},
				[9] = {
					relativePoint = "CENTER",
					coord_x = 200,
					coord_y = 50
				},
				[10] = {
					relativePoint = "CENTER",
					coord_x = 250,
					coord_y = 50
				},
				[11] = {
					relativePoint = "CENTER",
					coord_x = 300,
					coord_y = 50
				},
				[12] = {
					relativePoint = "CENTER",
					coord_x = 350,
					coord_y = 50
				},
				[13] = {
					relativePoint = "CENTER",
					coord_x = 400,
					coord_y = 50
				}
			}
        },
		-- This is to make some defaults for old diseases module, should be removed together with the code for upgrade from < 2.0.0.
		diseases = {
            enabled = true,
            texture_pack = "Blizzard",
            uiScale = 1.0,
            showCooldown = true,
            showText = true,
            subSecondPrescision = true,
            subSecondPrescisionThreshold = 3.5,
            showEbonPlague = true,
            showScarletFever = false,
			showShadowInfusion = true,
			showDarkTransformation = true,
			showBladeBarrier = true,
			showNecroticStrike = true,
			showSuddenDoom = true,
			showKillingMachine = true,
			showRime = true,
            fallOffPulse = 5,
            [1] = {
                relativePoint = "CENTER",
                coord_x = 65,
                coord_y = -160
            },
            [2] = {
                relativePoint = "CENTER",
                coord_x = -65,
                coord_y = -160
            },
            [3] = {
                relativePoint = "CENTER",
                coord_x = 0,
                coord_y = -200
            },
            [4] = {
                relativePoint = "CENTER",
                coord_x = 50,
                coord_y = -200
            },
            [5] = {
                relativePoint = "CENTER",
                coord_x = 200,
                coord_y = 0
            },
            [6] = {
                relativePoint = "CENTER",
                coord_x = 250,
                coord_y = 0
            },
            [7] = {
                relativePoint = "CENTER",
                coord_x = 300,
                coord_y = 0
            },
            [8] = {
                relativePoint = "CENTER",
                coord_x = 350,
                coord_y = 0
            },
            [9] = {
                relativePoint = "CENTER",
                coord_x = 200,
                coord_y = 50
            },
            [10] = {
                relativePoint = "CENTER",
                coord_x = 250,
                coord_y = 50
            },
            [11] = {
                relativePoint = "CENTER",
                coord_x = 300,
                coord_y = 50
            }
        },
        runes = {
            enabled = true,
            hideBlizzardRuneFrame = true,
            texture_pack = "Blizzard",
			border_pack = "Blizzard",
            uiScale = 1.0,
            runeCooldownBrightness = 0.5,
            showCooldown = true,
            showText = true,
            runePriority = true,
            [1] = {
                relativePoint = "CENTER",
                coord_x = -30,
                coord_y = -100
            },
            [2] = {
                relativePoint = "CENTER",
                coord_x = -30,
                coord_y = -130
            },
            [3] = {
                relativePoint = "CENTER",
                coord_x = 30,
                coord_y = -100
            },
            [4] = {
                relativePoint = "CENTER",
                coord_x = 30,
                coord_y = -130
            },
            [5] = {
                relativePoint = "CENTER",
                coord_x = 0,
                coord_y = -100
            },
            [6] = {
                relativePoint = "CENTER",
                coord_x = 0,
                coord_y = -130
            },
        },
        runebars = {
            enabled = true,
            hideBlizzardRuneFrame = true,
            uiScale = 1.0,
            showText = false,
            barHeight = 40,
            barWidth = 25,
            barTexture = "Blizzard",
            barBorder = "Blizzard Tooltip",
            barVertical = true,
            borderSize = 2,
            barEdgeOffset = 6,
            [1] = {
                relativePoint = "CENTER",
                coord_x = -25,
                coord_y = -20
            },
            [2] = {
                relativePoint = "CENTER",
                coord_x = -25,
                coord_y = -60
            },
            [3] = {
                relativePoint = "CENTER",
                coord_x = 25,
                coord_y = -20
            },
            [4] = {
                relativePoint = "CENTER",
                coord_x = 25,
                coord_y = -60
            },
            [5] = {
                relativePoint = "CENTER",
                coord_x = 0,
                coord_y = -20
            },
            [6] = {
                relativePoint = "CENTER",
                coord_x = 0,
                coord_y = -60
            },
        },
        runicpower = {
            enabled = true,
            uiScale = 1.0,
            relativePoint = "CENTER",
            coord_x = 0,
            coord_y = -160,
        },
        presencecheck = {
            enabled = true,
            noPresenceWarning = true,
            specChangeWarning = true,
        },
        failstat = {
            enabled = true,
            logRuneStatistics = false
        }
    },
}

DKM.frames = {}

function DKM:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("DKMDB", defaults, "Default")
    
	if self.db.profile.general.version ~= nil then
		if self.db.profile.general.version.major ~= DKM.version.major
		or self.db.profile.general.version.minor ~= DKM.version.minor
		or self.db.profile.general.version.revision ~= DKM.version.revision then
			self:UpgradeProfile()
			self.db.profile.general.version = DKM.version
		end
	else
		self:UpgradeProfile()
		self.db.profile.general.version = DKM.version
	end
	
    self.options = options
    LibStub("AceConfig-3.0"):RegisterOptionsTable("DKM", options)
    self.optionsFrameAbout = LibStub("LibAboutPanel").new(nil, "DKM")
    self.optionsFrameGeneral = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "General", "DKM", "general")
    
    self:RegisterChatCommand("DKM","ChatCommand")
    
    self:AddTexturePacks()
end

function DKM:UpgradeProfile()
	-- Upgrade for version prior 2.0.0 (they had no version string in profile)
	if self.db.profile.general.version == nil then
		if self.db.profile.diseases ~= nil then
			self.db.profile.trackers.enabled = self.db.profile.diseases.enabled
            self.db.profile.trackers.uiScale = self.db.profile.diseases.uiScale
			self.db.profile.trackers.showCooldown = self.db.profile.diseases.showCooldown
			self.db.profile.trackers.showText = self.db.profile.diseases.showText
			self.db.profile.trackers.subSecondPrescision = self.db.profile.diseases.subSecondPrescision
			self.db.profile.trackers.subSecondPrescisionThreshold = self.db.profile.diseases.subSecondPrescisionThreshold
			self.db.profile.trackers.showEbonPlague = self.db.profile.diseases.showEbonPlague
			self.db.profile.trackers.showScarletFever = self.db.profile.diseases.showScarletFever
			self.db.profile.trackers.showShadowInfusion = self.db.profile.diseases.showShadowInfusion
			self.db.profile.trackers.showDarkTransformation = self.db.profile.diseases.showDarkTransformation
			self.db.profile.trackers.showBladeBarrier = self.db.profile.diseases.showBladeBarrier
			self.db.profile.trackers.showNecroticStrike = self.db.profile.diseases.showNecroticStrike
			self.db.profile.trackers.showSuddenDoom = self.db.profile.diseases.showSuddenDoom
			self.db.profile.trackers.showKillingMachine = self.db.profile.diseases.showKillingMachine
			self.db.profile.trackers.showRime = self.db.profile.diseases.showRime
			self.db.profile.trackers.fallOffPulse = self.db.profile.diseases.fallOffPulse
			
			for i = 1, 11 do
				self.db.profile.trackers.trackable[i] = self.db.profile.diseases[i]
			end
			
			self.db.profile.diseases = {}
		end
	end
end

function DKM:OnEnable()
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:UpdateWidgetsLocking()
    self:UpdateUIScale()
    self:UpdateUIOpacity()
    
    self:Print(L["Enabled. Version VERSION."](self.version))
    self:Print(L["Type /dkm in chat to open configuration window."])
    
    for name, module in self:IterateModules() do
        if module:IsEnabledOption() then
            module:Enable()
        end
        
        module:ToggleOptions()
    end
end

function DKM:OnDisable()
    self:UnregisterEvent("PLAYER_REGEN_ENABLED")
    self:UnregisterEvent("PLAYER_REGEN_DISABLED")
    self:Print(L["Disabled."])
end

function DKM:RegisterDragableFrame(frame, overlay_text)
    -- Record frame in global frame list to turn dragging on/off
    self.frames[#self.frames + 1] = frame
    Sticky:RegisterFrame( frame )
    
    if overlay_text then
        Sticky:SetFrameText( frame, ''..overlay_text )
    end
    
--    This will make frame stick only to other frames in specified group; might be useful moving this into options a bit later    
--    Sticky:SetFrameGroup( frame, 'DKMFrames' )
end

function DKM:EnableDragOnFrame(frame)
    frame:EnableMouse(true)
    frame:SetMovable(true)
    frame:RegisterForDrag("LeftButton")
    Sticky:SetFrameEnabled( frame, true )
end

function DKM:DisableDragOnFrame(frame)
    Sticky:SetFrameEnabled( frame, false )
    frame:RegisterForDrag("")
    frame:SetMovable(false)
    frame:EnableMouse(false)
    self:OnStopFrameMoving(frame)
end

function DKM:OnStopFrameMoving(frame)
    Sticky:AnchorFrame(frame)
    
    point, relativeTo, relativePoint, xOfs, yOfs = frame:GetPoint(1)
    frame.coords.relativePoint = relativePoint
    frame.coords.coord_x = xOfs
    frame.coords.coord_y = yOfs
end

function DKM:IsLockWidgets(info)
    return self.db.profile.general.lockWidgets
end

function DKM:ToggleLockWidgets(info, value)
    self.db.profile.general.lockWidgets = value
    self:UpdateWidgetsLocking()
end

function DKM:GetUIScale(info)
    return self.db.profile.general.uiScale
end

function DKM:SetUIScale(info, value)
    self.db.profile.general.uiScale = value
    self:UpdateUIScale()
end

function DKM:UpdateUIScale()
    for name, module in self:IterateModules() do
        module:UpdateUIScale()
    end
end

function DKM:GetUIOpacityInCombat(info)
    return self.db.profile.general.uiOpacityInCombat
end

function DKM:SetUIOpacityInCombat(info, value)
    self.db.profile.general.uiOpacityInCombat= value
    self:UpdateUIOpacity()
end

function DKM:GetUIOpacityOutOfCombat(info)
    return self.db.profile.general.uiOpacityOutOfCombat
end

function DKM:SetUIOpacityOutOfCombat(info, value)
    self.db.profile.general.uiOpacityOutOfCombat= value
    self:UpdateUIOpacity()
end

function DKM:PLAYER_REGEN_ENABLED()
    self:UpdateUIOpacityPerFrame(self.db.profile.general.uiOpacityOutOfCombat)
end

function DKM:PLAYER_REGEN_DISABLED()
    self:UpdateUIOpacityPerFrame(self.db.profile.general.uiOpacityInCombat)
end

function DKM:UpdateUIOpacity()
    if not InCombatLockdown() then
       self:UpdateUIOpacityPerFrame(self.db.profile.general.uiOpacityOutOfCombat)
    else
       self:UpdateUIOpacityPerFrame(self.db.profile.general.uiOpacityInCombat)
    end
end

function DKM:UpdateUIOpacityPerFrame(opacity)
    for i = 1,#self.frames do
        self.frames[i]:SetAlpha(opacity)
    end
end

function DKM:UpdateWidgetsLocking()
    -- Enable/disable drag on every frame
    for i = 1,#self.frames do
        if self.db.profile.general.lockWidgets then
            self:DisableDragOnFrame( self.frames[i] )
        else
            self:EnableDragOnFrame( self.frames[i] )
            -- We must show every frame that is dragable
            self.frames[i]:Show()
        end
    end

    -- Enable movement via LibStickyFrames
    if self.db.profile.general.lockWidgets then
        Sticky:SetGroup( false )
    else
        Sticky:SetGroup( true )
    end
    
    -- Run custom methods for every module
    for name, module in self:IterateModules() do
        module:UpdateWidgetsLocking()
    end
end

function DKM:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrameAbout)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(DKM,"dkm",input)
    end
end

function DKM:FormatTime(time, subSecondPrescision, threshold)
    time = max(time, 0)
    local timeMinutes = math.fmod(time, 3600)
    local timeSeconds = math.fmod(timeMinutes, 60)

    if ( subSecondPrescision == false
    or   time > threshold ) then
        return floor(timeSeconds)
    end

    return floor(timeSeconds).."."..floor(10 * math.fmod(timeSeconds, 1))
end

function DKM:AddTexturePacks()
    self.gfxPacksRunesTextures["Orbs"] = "Orbs"
	self.gfxPackRunesTexturesData["Orbs"] = {
        path = "Interface\\Addons\\DKM\\art\\runes\\textures\\orbs",
        form = "circle"
    }
	
	self.gfxPacksRunesBorders["None"] = "None"
	self.gfxPackRunesBordersData["None"] = {
        path = "Interface\\Addons\\DKM\\art\\runes\\borders\\none",
        form = "circle"
    }
	
	self.gfxPacksRunesBorders["Runewatch"] = "Runewatch"
	self.gfxPackRunesBordersData["Runewatch"] = {
        path = "Interface\\Addons\\DKM\\art\\runes\\borders\\runewatch",
        form = "circle"
    }
	
	self.gfxPacksTrackersTextures["Circles"] = "Circles"
	self.gfxPackTrackersTexturesData["Circles"] = {
        path = "Interface\\Addons\\DKM\\art\\trackers\\textures\\circles",
        form = "circle"
    }
	
	self.gfxPacksTrackersBorders["Black Circle"] = "Black Circle"
	self.gfxPackTrackersBordersData["Black Circle"] = {
        path = "Interface\\Addons\\DKM\\art\\trackers\\borders\\black circle",
        form = "circle"
    }
	
	self.gfxPacksTrackersBorders["Runewatch"] = "Runewatch"
	self.gfxPackTrackersBordersData["Runewatch"] = {
        path = "Interface\\Addons\\DKM\\art\\trackers\\borders\\runewatch",
        form = "circle"
    }
end
