--[[
    Title: DKM, Diseases module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])

local Diseases = DKM:NewModule("DKM_Diseases", nil, "AceConsole-3.0", "AceEvent-3.0")

local DiseaseBorderTextureName = "\\UI-Tooltip-Border"

local DiseasesData = {
    [1] = {
        spell_id = 55078,
        spell_name = "",
        texture_name = "\\Spell_DeathKnight_BloodPlague",
    },
    [2] = {
        spell_id = 55095,
        spell_name = "",
        texture_name = "\\Spell_DeathKnight_FrostFever",
    },
    [3] = {
        spell_id = 65142,
        spell_name = "",
        texture_name = "\\Ability_Creature_Cursed_03",
    }
}

DISEASE_BP = 1
DISEASE_FF = 2
DISEASE_EP = 3

Diseases.diseaseFrames = { }

function Diseases:OnInitialize()
    DKM.options.args.diseases.args.texture_pack = {
        name = L["Texture pack to use"],
        desc = L["One of texture packs located in 'art' directory."],
        type = "select",
        style = "dropdown",
        handler = Diseases,
        get = "GetTexturePack",
        set = "SetTexturePack",
        values = DKM.texturePacks
    }
    DKM.options.args.diseases.args.uiScale = {
        type = "range",
        name = L["Diseases UI Scale"],
        desc = L["Scales the diseases UI to a factor of the default."],
        min = 0.1,
        max = 2,
        step = 0.05,
        handler = Diseases,
        get = "GetUIScale",
        set = "SetUIScale",
    }
    DKM.options.args.diseases.args.showCooldown = {
        type = "toggle",
        width = "full",
        name = L["Show animated timer cooldown"],
        desc = L["Shows animated cooldown on disease widget when timer is active."],
        handler = Diseases,
        get = "IsShowCooldown",
        set = "SetShowCooldown",
    }
    DKM.options.args.diseases.args.showText = {
        type = "toggle",
        width = "full",
        name = L["Show timer text"],
        desc = L["Shows text on disease widget when timer is active."],
        handler = Diseases,
        get = "IsShowText",
        set = "SetShowText",
    }
    DKM.options.args.diseases.args.showEbonPlague = {
        type = "toggle",
        width = "full",
        name = L["Show Ebon Plague"],
        desc = L["Shows Ebon Plague disease on target."],
        handler = Diseases,
        get = "IsShowEbonPlague",
        set = "SetShowEbonPlague",
    }
    DKM.options.args.diseases.args.subSecondPrescision = {
        type = "toggle",
        width = "full",
        name = L["Sub-second timer prescision"],
        desc = L["Allows disease timers to show value with sub-second prescision."],
        handler = Diseases,
        get = "GetSubSecondPrescision",
        set = "SetSubSecondPrescision",
    }
    DKM.options.args.diseases.args.subSecondPrescisionThreshold = {
        type = "range",
        name = L["Sub-second timer prescision threshold"],
        desc = L["Time left on timer when sub-second prescision activates."],
        min = 0.5,
        max = 10,
        step = 0.1,
        handler = Diseases,
        get = "GetSubSecondPrescisionThreshold",
        set = "SetSubSecondPrescisionThreshold",
    }

    for disease_id = 1,#DiseasesData do
        self:LookupSpellName(disease_id)
        self:CreateDiseaseFrame(disease_id)
    end
end

function Diseases:OnEnable()
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_ALIVE")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    self:UpdateAllDiseaseTextures()
    self:UpdateDiseases()
    self:UpdateEbonPlagueVisibility()

    DKM:Print(L["Diseases module enabled."])
end

function Diseases:OnDisable()
    self:UnregisterEvent("PLAYER_TARGET_CHANGED")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:UnregisterEvent("PLAYER_ALIVE")
    self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    for disease_id = 1,#DiseasesData do
        self:UpdateMissingDisease(disease_id)
    end

    DKM:Print(L["Diseases module disabled."])
end

-- Spellnames differ from locale to locale
function Diseases:LookupSpellName(disease_id)
    DiseasesData[disease_id].spell_name = GetSpellInfo(DiseasesData[disease_id].spell_id)
end

function Diseases:CreateDiseaseFrame(disease_id)
    frame = CreateFrame("FRAME", "DiseaseFrame"..disease_id, UIParent)
    frame:SetPoint("CENTER", UIParent, DKM.db.profile.diseases[disease_id].relativePoint, DKM.db.profile.diseases[disease_id].coord_x, DKM.db.profile.diseases[disease_id].coord_y)
    frame:SetWidth(36)
    frame:SetHeight(36)
    frame:SetFrameLevel(0)
    frame.coords = DKM.db.profile.diseases[disease_id]

    texture = frame:CreateTexture("DiseaseTexture"..disease_id, "ARTWORK")
    texture:SetAllPoints(frame)
    texture:SetVertexColor(0.8,0.8,0.8)
    frame.texture = texture

    local cooldown = CreateFrame("COOLDOWN", "DiseaseCooldown", frame)
    cooldown:SetPoint("CENTER", texture, "CENTER")
    cooldown:SetWidth(25)
    cooldown:SetHeight(25)
    cooldown:SetFrameLevel(1)
    -- OmniCC Fix
    cooldown.noCooldownCount = true
    frame.cooldown = cooldown

    local border = CreateFrame("FRAME", "DiseaseFrameBorder", frame)
    border:SetAllPoints(frame)
    border:SetFrameLevel(4)
    frame.border = border

    local border_texture = border:CreateTexture("DiseaseBorderTexture"..disease_id, "OVERLAY")
    border_texture:SetAllPoints(border)
    border.texture = border_texture

    local text = border:CreateFontString("DiseaseText"..disease_id,"OVERLAY","DKM_UIFont16")
    text:SetAllPoints(border)
    text:SetTextColor(1,1,0,0.7)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetText("")
    frame.text = text

    frame.disease_id = disease_id
    frame.expire_time = 0
    frame.time_since_last_update = 0
    frame.active = false
    frame.found = false
    if (DKM.db.profile.general.lockWidgets) then
        frame:Hide()
    end

    DKM:RegisterDragableFrame(frame)
    self.diseaseFrames[disease_id] = frame
end

function Diseases:UpdateDiseases()
    for i = 1,40 do
        local nm, rank, icon, count, debuff, duration, expire, owner, isSteal = UnitAura("TARGET", i, "HARMFUL")
        if ( owner == "player"
        and  debuff == "Disease" ) then
            for disease_id = 1,#DiseasesData do
                self:UpdateDisease(nm, expire, duration, disease_id)
            end
        end
    end

    for disease_id = 1,#DiseasesData do
        self:UpdateMissingDisease(disease_id)
        self.diseaseFrames[disease_id].found = false
    end

    self:UpdateAllDiseaseTimerVisibility()
end

function Diseases:UpdateDisease(nm, expire, duration, disease_id)
    if nm ~= DiseasesData[disease_id].spell_name then
        return
    end

    self.diseaseFrames[disease_id].expire_time = expire
    self.diseaseFrames[disease_id].text:SetText( self:FormatTime(expire - GetTime()) )
    self.diseaseFrames[disease_id].cooldown:SetCooldown(expire - duration, duration)
    self.diseaseFrames[disease_id].found = true
    self.diseaseFrames[disease_id].active = true
    self.diseaseFrames[disease_id]:Show()

    if self.diseaseFrames[disease_id]:GetScript("OnUpdate") == nil then
        self.diseaseFrames[disease_id]:SetScript("OnUpdate", function(frame, elapsed) Diseases:DiseaseOnUpdate(frame, elapsed) end)
    end
end

function Diseases:UpdateMissingDisease(disease_id)
    if self.diseaseFrames[disease_id].found == false then
        self.diseaseFrames[disease_id].active = false

        if self.diseaseFrames[disease_id]:GetScript("OnUpdate") ~= nil then
            self.diseaseFrames[disease_id]:SetScript("OnUpdate", nil)
        end

        self.diseaseFrames[disease_id].text:SetText("")
        if DKM.db.profile.general.lockWidgets then
            self.diseaseFrames[disease_id]:Hide()
        end
    end
end

function Diseases:DiseaseOnUpdate(frame, elapsed)
    frame.time_since_last_update = frame.time_since_last_update + elapsed

    if frame.time_since_last_update > 0.1 then
        frame.time_since_last_update = 0

        frame.text:SetText( self:FormatTime(frame.expire_time - GetTime()) )
    end
end

function Diseases:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
    local _, ev, src, _, _, dst, _, _, a1, a2, a3, a4, a5, a6, a7, a8 = ...

    if src == UnitGUID("player") then
        if ev == "SPELL_AURA_APPLIED"
        or ev == "SPELL_AURA_REMOVED"
        or ev == "SPELL_AURA_REFRESH" then
            self:UpdateDiseases()
        end
    end
end

function Diseases:PLAYER_ENTERING_WORLD(event, ...)
   self:UpdateDiseases()
end

function Diseases:PLAYER_ALIVE(event, ...)
    self:UpdateDiseases()
end

function Diseases:PLAYER_TARGET_CHANGED(event, ...)
    self:UpdateDiseases()
end

function Diseases:GetTexturePack(info)
    return DKM.db.profile.diseases.texture_pack
end

function Diseases:SetTexturePack(info, value)
    DKM.db.profile.diseases.texture_pack = value
    self:UpdateAllDiseaseTextures()
end

function Diseases:UpdateAllDiseaseTextures()
    for disease_id = 1,#DiseasesData do
        self.diseaseFrames[disease_id].texture:SetTexture(DKM.texturePackData[DKM.db.profile.diseases.texture_pack].diseases..DiseasesData[disease_id].texture_name)
        self.diseaseFrames[disease_id].border.texture:SetTexture()
        self.diseaseFrames[disease_id].border.texture:SetTexture(DKM.texturePackData[DKM.db.profile.diseases.texture_pack].diseases..DiseaseBorderTextureName)

        self.diseaseFrames[disease_id].cooldown:SetWidth(
            self.diseaseFrames[disease_id]:GetWidth()
            * DKM.textureFormCooldownScaleFactor[ DKM.texturePackData[DKM.db.profile.diseases.texture_pack].form ]
        )
        self.diseaseFrames[disease_id].cooldown:SetHeight(
            self.diseaseFrames[disease_id]:GetHeight()
            * DKM.textureFormCooldownScaleFactor[ DKM.texturePackData[DKM.db.profile.diseases.texture_pack].form ]
        )
    end
end

function Diseases:GetUIScale(info)
    return DKM.db.profile.diseases.uiScale
end

function Diseases:SetUIScale(info, value)
    DKM.db.profile.diseases.uiScale = value
    self:UpdateUIScale()
end

function Diseases:UpdateUIScale()
    for disease_id = 1,#self.diseaseFrames do
        self.diseaseFrames[disease_id]:SetScale( DKM.db.profile.general.uiScale * DKM.db.profile.diseases.uiScale )
    end
end

function Diseases:UpdateWidgetsLocking()
    -- Update diseases if we locked widgets - this will make them disapear if needed
    if DKM.db.profile.general.lockWidgets then
        self:UpdateDiseases()
    end
end

function Diseases:GetSubSecondPrescision(info)
    return DKM.db.profile.diseases.subSecondPrescision
end

function Diseases:SetSubSecondPrescision(info, value)
    DKM.db.profile.diseases.subSecondPrescision = value
end

function Diseases:GetSubSecondPrescisionThreshold(info)
    return DKM.db.profile.diseases.subSecondPrescisionThreshold
end

function Diseases:SetSubSecondPrescisionThreshold(info, value)
    DKM.db.profile.diseases.subSecondPrescisionThreshold = value
end

function Diseases:IsShowCooldown(info)
    return DKM.db.profile.diseases.showCooldown
end

function Diseases:SetShowCooldown(info, value)
    DKM.db.profile.diseases.showCooldown = value
    self:UpdateAllDiseaseTimerVisibility()
end

function Diseases:IsShowEbonPlague(info)
    return DKM.db.profile.diseases.showEbonPlague
end

function Diseases:SetShowEbonPlague(info, value)
    DKM.db.profile.diseases.showEbonPlague = value
    self:UpdateEbonPlagueVisibility()
end

function Diseases:UpdateEbonPlagueVisibility()
    if DKM.db.profile.diseases.showEbonPlague then
        self.diseaseFrames[DISEASE_EP]:Show()
    else
        self.diseaseFrames[DISEASE_EP]:Hide()
    end
end

function Diseases:IsShowText(info)
    return DKM.db.profile.diseases.showText
end

function Diseases:SetShowText(info, value)
    DKM.db.profile.diseases.showText = value
    self:UpdateAllDiseaseTimerVisibility()
end

function Diseases:UpdateAllDiseaseTimerVisibility()
    for disease_id = 1,#DiseasesData do
        if DKM.db.profile.diseases.showCooldown then
            self.diseaseFrames[disease_id].cooldown:Show()
        else
            self.diseaseFrames[disease_id].cooldown:Hide()
        end

        if DKM.db.profile.diseases.showText then
            self.diseaseFrames[disease_id].text:Show()
        else
            self.diseaseFrames[disease_id].text:Hide()
        end
    end
end

function Diseases:FormatTime(time)
    return DKM:FormatTime(time, DKM.db.profile.diseases.subSecondPrescision, DKM.db.profile.diseases.subSecondPrescisionThreshold)
end
