--[[
    Title: DKM, Runes module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBase, L["Unable to locate ModuleBase object."])

ModuleBaseRunes = {}

setmetatable( ModuleBaseRunes, { __index = ModuleBase } )

function ModuleBaseRunes:InitializeRunes()
    for rune_id = 1,6 do
        self.runeIDsToFrameIDs[rune_id] = rune_id
        self.runeFrames[rune_id] = self:CreateRuneFrame( rune_id )
    end
end

function ModuleBaseRunes:RUNE_POWER_UPDATE( event, rune_id )
    self:UpdateRuneCooldownByRuneID( rune_id )
end

function ModuleBaseRunes:RUNE_TYPE_UPDATE( event, rune_id )
    self:UpdateRuneTypeByRuneID( rune_id )
end

function ModuleBaseRunes:PLAYER_ALIVE( event, ... )
    self:UpdateRunes()
end

function ModuleBaseRunes:PLAYER_ENTERING_WORLD( event, ... )
    self:UpdateRunes()
end

function ModuleBaseRunes:UpdateRunes()
    for frame_id, frame in pairs( self.runeFrames ) do
        self:UpdateRuneCooldownByFrame( frame.real_frame )
        self:UpdateRuneTypeByFrame( frame.real_frame )
    end
end

function ModuleBaseRunes:UpdateRuneCooldownByRuneID( rune_id )
    self:UpdateRuneCooldownByFrame( self:GetRealRuneFrameByRuneID( rune_id ) )
end

function ModuleBaseRunes:UpdateRuneTypeByRuneID( rune_id )
    self:UpdateRuneTypeByFrame( self:GetRealRuneFrameByRuneID( rune_id ) )
end

function ModuleBaseRunes:GetRuneTimeLeft( start, duration )
    return duration - ( GetTime() - start )
end

function ModuleBaseRunes:DeduceFrameIDFromRuneID( rune_id )
    return self.runeIDsToFrameIDs[ rune_id ]
end

function ModuleBaseRunes:GetRealRuneFrameByRuneID( rune_id )
    return self.runeFrames[ self:DeduceFrameIDFromRuneID( rune_id ) ].real_frame
end

function ModuleBaseRunes:SwapFrames( rune_id )
    local real_frame = self.runeFrames[rune_id].real_frame
    
    self.runeFrames[rune_id].real_frame = self.runeFrames[rune_id + 1].real_frame
    self.runeFrames[rune_id].real_frame:SetAllPoints( self.runeFrames[rune_id] )
    self.runeFrames[rune_id + 1].real_frame = real_frame
    self.runeFrames[rune_id + 1].real_frame:SetAllPoints( self.runeFrames[rune_id + 1] )
    
    local id = self.runeIDsToFrameIDs[rune_id]
    self.runeIDsToFrameIDs[rune_id] = self.runeIDsToFrameIDs[rune_id + 1]
    self.runeIDsToFrameIDs[rune_id + 1] = id
end

function ModuleBaseRunes:IsShowText(info)
    return self.db.showText
end

function ModuleBaseRunes:SetShowText(info, value)
    self.db.showText = value
    self:UpdateAllRuneTimerVisibility()
end

function ModuleBaseRunes:IsHideBlizzardRuneFrame(info)
    return self.db.hideBlizzardRuneFrame
end

function ModuleBaseRunes:ToggleHideBlizzardRuneFrame(info, value)
    self.db.hideBlizzardRuneFrame = value
    self:UpdateBlizzardRuneFrame()
end

function ModuleBaseRunes:UpdateBlizzardRuneFrame()
    if self.db.hideBlizzardRuneFrame then
        _G["RuneFrame"]:Hide()
    else
        _G["RuneFrame"]:Show()
    end
end
