--[[
    Title: DKM, Presence Check module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBase, L["Unable to locate ModuleBase object."])

local PresenceCheck = DKM:NewModule("PresenceCheck", ModuleBase, "AceConsole-3.0", "AceEvent-3.0")

function PresenceCheck:OnInitialize()
    DKM.options.args.presencecheck = {
        name = "General",
        type = "group",
        desc = "General options",
        args = {
            noPresenceWarning = {
                type = "toggle",
                width = "full",
                name = L["Warn when you have no presence active"],
                desc = L["Warns you if you have no presence active when you enter combat."],
                handler = PresenceCheck,
                get = "IsNoPresenceWarning",
                set = "ToggleNoPresenceWarning",
            },
            specChangeWarning = {
                type = "toggle",
                width = "full",
                name = L["Warn when you change talent specs"],
                desc = L["Warns you when you change between primary and secondary talent specs (dual spec)."],
                handler = PresenceCheck,
                get = "IsSpecChangeWarning",
                set = "ToggleSpecChangeWarning",
            }
        }
    }
    
    self.db = DKM.db.profile.presencecheck
    self.options = DKM.options.args.presencecheck
    self:InitBaseOptions(L["Enables "..self:GetName().." module."])
    DKM.optionsFramePresenceCheck = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "Presence Check", "DKM", "presencecheck")
    
    self:SetEnabledState(false)
end

function PresenceCheck:OnEnable()
    self:UpdateNoPresenceWarning()
    self:UpdateSpecChangeWarning()

    DKM:Print(L[self:GetName().." module enabled."])
end

function PresenceCheck:OnDisable()
    self:UnregisterEvent("PLAYER_REGEN_DISABLED")
    self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")

    DKM:Print(L[self:GetName().." module disabled."])
end

function PresenceCheck:IsNoPresenceWarning(info)
    return self.db.noPresenceWarning
end

function PresenceCheck:ToggleNoPresenceWarning(info, value)
    self.db.noPresenceWarning = value
    self:UpdateNoPresenceWarning()
end

function PresenceCheck:IsSpecChangeWarning(info)
    return self.db.specChangeWarning
end

function PresenceCheck:ToggleSpecChangeWarning(info, value)
    self.db.specChangeWarning = value
    self:UpdateSpecChangeWarning()
end

function PresenceCheck:UpdateNoPresenceWarning()
    if self.db.noPresenceWarning then
        self:RegisterEvent("PLAYER_REGEN_DISABLED")
    else
        self:UnregisterEvent("PLAYER_REGEN_DISABLED")
    end
end

function PresenceCheck:UpdateSpecChangeWarning()
    if self.db.specChangeWarning then
        self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
    else
        self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
    end
end

function PresenceCheck:ACTIVE_TALENT_GROUP_CHANGED()
    self:PerformCheck()
end

function PresenceCheck:PLAYER_REGEN_DISABLED()
    self:PerformCheck()
end

function PresenceCheck:PerformCheck()
    if ( UnitBuff("player", "Blood Presence") == nil
    and  UnitBuff("player", "Frost Presence") == nil
    and  UnitBuff("player", "Unholy Presence") == nil ) then
        RaidNotice_AddMessage(RaidWarningFrame, L["NO PRESENCE ACTIVE!"], { r = 1, g = 0, b = 0 })
        PlaySound("RaidWarning")
    end
end
