--[[
    Title: DKM, Runes module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")
local LSM = LibStub( "LibSharedMedia-3.0" )

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBaseRunes, L["Unable to locate ModuleBaseRunes object."])

local RuneBars = DKM:NewModule("RuneBars", ModuleBaseRunes, "AceConsole-3.0", "AceEvent-3.0")

local RuneTextureColor = {
    [false] = {
        [1] = { 0.8, 0.1, 0.1, 0.5 },
        [2] = { 0.5, 0.8, 0.1, 0.5 },
        [3] = { 0, 0.3, 0.8, 0.5 },
        [4] = { 0.55, 0.1, 0.55, 0.5 }
    },
    [true] = {
        [1] = { 0.8, 0.1, 0.1, 1 },
        [2] = { 0.5, 0.8, 0.1, 1 },
        [3] = { 0, 0.3, 0.8, 1 },
        [4] = { 0.55, 0.1, 0.55, 1 }
    }
}

local RuneTextColor = {
    [false] = {
        [1] = { 0.8, 0.1, 0.1, 1 },
        [2] = { 0.5, 0.8, 0.1, 1 },
        [3] = { 0, 0.3, 0.8, 1 },
        [4] = { 0.55, 0.1, 0.55, 1 }
    },
    [true] = {
        [1] = { 0.8, 0.1, 0.1, 1 },
        [2] = { 0.5, 0.8, 0.1, 1 },
        [3] = { 0, 0.3, 0.8, 1 },
        [4] = { 0.55, 0.1, 0.55, 1 }
    }
}

RuneBars.runeFrames = {}
RuneBars.runeIDsToFrameIDs = {}

function RuneBars:OnInitialize()
    DKM.options.args.runebars = {
        name = "General",
        type = "group",
        desc = "General options",
        args = {
            hideBlizzardRuneFrame = {
                type = "toggle",
                width = "full",
                name = L["Hide Blizzard Rune Frame"],
                desc = L["Hides Blizzard Rune Frame."],
                handler = RuneBars,
                get = "IsHideBlizzardRuneFrame",
                set = "ToggleHideBlizzardRuneFrame",
            },
            uiScale = {
                type = "range",
                name = L["Runes UI Scale"],
                desc = L["Scales the runes UI to a factor of the default."],
                min = 0.1,
                max = 2,
                step = 0.05,
                handler = RuneBars,
                get = "GetUIScale",
                set = "SetUIScale",
                order = -1
            },
            showText = {
                type = "toggle",
                width = "full",
                name = L["Show timer text"],
                desc = L["Shows text on rune widget when timer is active."],
                handler = RuneBars,
                get = "IsShowText",
                set = "SetShowText",
            },
            barHeight = {
                type = "range",
                name = L["Rune bar height"],
                desc = L["Height of the Rune bars."],
                min = 1,
                max = 100,
                step = 1,
                handler = RuneBars,
                get = "GetBarHeight",
                set = "SetBarHeight",
            },
            barWidth = {
                type = "range",
                name = L["Rune bar width"],
                desc = L["Width of the Rune bars."],
                min = 1,
                max = 100,
                step = 1,
                handler = RuneBars,
                get = "GetBarWidth",
                set = "SetBarWidth",
            },
            barTexture = {
                type = "select",
                name = L["Bar Texture"],
                desc = L["Texture that will be used for rune bars."],
                values = LSM:HashTable("statusbar"),
                dialogControl = "LSM30_Statusbar",
                order = 101,
                handler = RuneBars,
                get = "GetBarTexture",
                set = "SetBarTexture",
            },
            barBorder = {
                type = "select",
                name = L["Border Texture"],
                desc = L["Texture that will be used for rune bars border."],
                values = LSM:HashTable("border"),
                dialogControl = "LSM30_Border",
                order = 102,
                handler = RuneBars,
                get = "GetBarBorder",
                set = "SetBarBorder",
            },
            barVertical = {
                type = "toggle",
                width = "full",
                name = L["Vertical bars"],
                desc = L["Bars will grow vertically if selected, horizontally otherwise."],
                handler = RuneBars,
                get = "IsBarVertical",
                set = "SetBarVertical",
            },
            borderSize = {
                type = "range",
                name = L["Border size"],
                desc = L["Border size of rune bars."],
                min = 1,
                max = 8,
                step = 1,
                order = 103,
                handler = RuneBars,
                get = "GetBorderSize",
                set = "SetBorderSize",
            },
            barEdgeOffset = {
                type = "range",
                name = L["Bar edge offset"],
                desc = L["Offset from edge to the actualt bar texture."],
                min = 0,
                max = 32,
                step = 1,
                order = 104,
                handler = RuneBars,
                get = "GetBarEdgeOffset",
                set = "SetBarEdgeOffset",
            }
        }
    }
    
    self.db = DKM.db.profile.runebars
    self.options = DKM.options.args.runebars
    self:InitBaseOptions(L["Enables "..self:GetName().." module."])
    DKM.optionsFrameRuneBars = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "RuneBars", "DKM", "runebars")
    
    self:InitializeRunes()
    
    self:SetEnabledState(false)
end

function RuneBars:OnEnable()
    self:UpdateBlizzardRuneFrame()

    self:RegisterEvent("RUNE_TYPE_UPDATE")
    self:RegisterEvent("RUNE_POWER_UPDATE")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_ALIVE")
    
    for frame_id, frame in pairs( self.runeFrames ) do
        self:UpdateRuneCooldownByFrame( frame.real_frame, true )
        self:UpdateRuneTypeByFrame( frame.real_frame )
        frame:Show()
    end
    
    self:UpdateBarsSize()
    self:UpdateBarBorders()
    self:UpdateBarTextures()
    self:UpdateAllRuneTimerVisibility()
    
    DKM:Print(L[self:GetName().." module enabled."])
end

function RuneBars:OnDisable()
    self:UnregisterEvent("RUNE_TYPE_UPDATE")
    self:UnregisterEvent("RUNE_POWER_UPDATE")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:UnregisterEvent("PLAYER_ALIVE")
    
    for frame_id, frame in pairs( self.runeFrames ) do
        frame:Hide()
    end
    
    DKM:Print(L[self:GetName().." module disabled."])
end

function RuneBars:CreateRuneFrame(rune_id)
    local frame_base = CreateFrame( "FRAME", self:GetName().."FrameBase"..rune_id, UIParent )
    frame_base:SetPoint( "CENTER", UIParent, DKM.db.profile.runebars[rune_id].relativePoint, DKM.db.profile.runebars[rune_id].coord_x, DKM.db.profile.runebars[rune_id].coord_y )
    frame_base:SetWidth(25)
    frame_base:SetHeight(40)
    frame_base:SetFrameLevel(1)
    frame_base.coords = DKM.db.profile.runebars[rune_id]
    frame_base:Hide()
    
    local frame_real = CreateFrame( "FRAME", self:GetName().."FrameReal"..rune_id, frame_base )
    frame_real:SetAllPoints(frame_base)
    frame_real:SetFrameLevel(1)
    frame_base.real_frame = frame_real
    
    local frame_texture = CreateFrame( "FRAME", self:GetName().."FrameTexture"..rune_id, frame_real )
    frame_texture:SetPoint("CENTER")
    frame_texture:SetFrameLevel(2)
    frame_texture:SetWidth( frame_base:GetWidth() )
    frame_texture:SetHeight( frame_base:GetHeight() )
    frame_real.texture_frame = frame_texture
    
    local texture = frame_texture:CreateTexture( self:GetName().."Texture"..rune_id, "ARTWORK" )
    texture:SetTexture( LSM:Fetch( "statusbar", self.db.barTexture ) )
    frame_real.texture = texture
    
    local frame_text = CreateFrame( "FRAME", self:GetName().."FrameText"..rune_id, frame_real )
    frame_text:SetAllPoints(frame_real)
    frame_text:SetFrameLevel(3)
    frame_real.text_frame = frame_text
    
    local text = frame_text:CreateFontString( self:GetName().."Text"..rune_id, "OVERLAY", "DKM_UIFont16" )
    text:SetAllPoints(frame_text)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetText("")
    frame_real.text = text
    
    frame_real.max_value = 100
    frame_real.min_value = 0
    frame_real.rune_id = rune_id
    frame_real.cd = 0
    self:SetBorderForFrame( frame_base )
    -- The following method is called inside SetBorderForFrame()
    -- self:UpdateBar( frame_real )
    DKM:RegisterDragableFrame( frame_base, rune_id )
    return frame_base
end

function RuneBars:SetBorderForFrame(frame)
    if self.db.borderSize ~= 0 then
        frame:SetBackdrop(
            {
                bgFile = "Interface\\Buttons\\WHITE8X8",
                edgeFile = LSM:Fetch( "border", self.db.barBorder ),
                tile = false,
                tileSize = 8,
                edgeSize = self.db.borderSize * 4,
                insets = {
                    left = self.db.borderSize,
                    right = self.db.borderSize,
                    top = self.db.borderSize,
                    bottom = self.db.borderSize
                }
            }
        )
        frame:SetBackdropBorderColor( 0.5, 0.5, 0.5, 1 )
        frame:SetBackdropColor( 0.1, 0.1, 0.1, 1 )
    else
        frame:SetBackdrop( nil )
    end
    
    frame.real_frame.texture_frame:SetHeight( frame:GetHeight() - self.db.barEdgeOffset )
    frame.real_frame.texture_frame:SetWidth( frame:GetWidth() - self.db.barEdgeOffset )
    self:UpdateBar( frame.real_frame )
end

function RuneBars:UpdateBar( frame )
    local fraction
    local range = frame.max_value - frame.min_value
    local value = frame.max_value - frame.cd
    
    if range > 0 and value > 0 and range >= value then
        fraction = value / range
    else
        fraction = range / 1000000
    end
    
    if self.db.barVertical then
        frame.texture:ClearAllPoints()
        frame.texture:SetPoint( "BOTTOMLEFT" )
        frame.texture:SetPoint( "BOTTOMRIGHT" )
        frame.texture:SetHeight( ( self.db.barHeight - self.db.barEdgeOffset ) * fraction )
        frame.texture:SetWidth( self.db.barWidth - self.db.barEdgeOffset )
        frame.texture:SetTexCoord( 0, 1, 1 - fraction, 1 )
    else
        frame.texture:ClearAllPoints()
        frame.texture:SetPoint( "TOPLEFT" )
        frame.texture:SetPoint( "BOTTOMLEFT" )
        frame.texture:SetHeight( self.db.barHeight - self.db.barEdgeOffset )
        frame.texture:SetWidth( ( self.db.barWidth - self.db.barEdgeOffset ) * fraction ) 
        frame.texture:SetTexCoord( 0, fraction, 0, 1 )
    end
end

function RuneBars:UpdateRuneColor( frame, ready )
    local runeType = GetRuneType( frame.rune_id )
    
    frame.texture:SetVertexColor( unpack( RuneTextureColor[ ready ][ runeType ] ) )
    frame.text:SetTextColor( unpack( RuneTextColor[ ready ][ runeType ] ) )
end

function RuneBars:UpdateRuneTypeByFrame( frame )
    local start, duration, runeReady = GetRuneCooldown( frame.rune_id )
    
    self:UpdateRuneColor( frame, runeReady )
end

function RuneBars:UpdateRuneCooldownByFrame( frame, on_enable )
    local start, duration, runeReady = GetRuneCooldown( frame.rune_id )
    local new_cd
    local old_cd = frame.cd
    
    if not runeReady then
        new_cd = self:GetRuneTimeLeft( start, duration )
        if new_cd < 0 then
            new_cd = 0
        end
    else
        new_cd = 0
    end
    
    if new_cd == frame.cd then
        return
    end
    
    frame.cd = new_cd
    
    frame.max_value = duration
    frame.min_value = 0
    
    if new_cd > 0 then
        if self:IsShowText() and frame.text:GetText() ~= ""..ceil( frame.cd ) then
            frame.text:SetText( ceil( frame.cd ) )
        end
        
        if new_cd > old_cd then
            self:UpdateRuneColor( frame, false )
            frame:SetScript( "OnUpdate", function(self, elapsed) RuneBars:UpdateRuneCooldownByFrame(self) end )
        end
    else
        frame.text:SetText( "" )
        self:UpdateRuneColor( frame, true )
        frame:SetScript( "OnUpdate", nil )
    end
    
    self:UpdateBar( frame )
    
    if not on_enable then
        self:UpdateRunePriority( frame.rune_id )
    end
end

function RuneBars:UpdateRunePriority( rune_id )
    if rune_id % 2 == 0 then
        rune_id = rune_id - 1
    end

    -- If this is removed, it errors out. Dunno why, gotta debug it. Leave it for now as it is.
    if self.runeFrames[rune_id + 1] == nil then
        return
    end

    -- Reverse priority for bars
    if self.runeFrames[rune_id].real_frame.cd > self.runeFrames[rune_id + 1].real_frame.cd then
        return
    end

    self:SwapFrames( rune_id )
end

function RuneBars:GetBarHeight(info)
    return self.db.barHeight
end

function RuneBars:SetBarHeight(info, value)
    self.db.barHeight = value
    self:UpdateBarsSize()
end

function RuneBars:GetBarWidth(info)
    return self.db.barWidth
end

function RuneBars:SetBarWidth(info, value)
    self.db.barWidth = value
    self:UpdateBarsSize()
end

function RuneBars:UpdateBarsSize()
    for frame_id, frame in pairs( self.runeFrames ) do
        self:UpdateBarSize( frame )
    end
end

function RuneBars:UpdateBarSize( frame )
    frame:SetHeight( self.db.barHeight )
    frame:SetWidth( self.db.barWidth )
    self:SetBorderForFrame( frame )
end

function RuneBars:GetUIScale(info)
    return self.db.uiScale
end

function RuneBars:SetUIScale(info, value)
    self.db.uiScale = value
    self:UpdateUIScale()
end

function RuneBars:UpdateUIScale()
    for frame_id, frame in pairs( self.runeFrames ) do
        frame:SetScale( DKM.db.profile.general.uiScale * self.db.uiScale )
    end
end

function RuneBars:GetBarTexture(info)
    return self.db.barTexture
end

function RuneBars:SetBarTexture(info, value)
    self.db.barTexture = value
    self:UpdateBarTextures()
end

function RuneBars:UpdateBarTextures()
    for frame_id, frame in pairs( self.runeFrames ) do
        frame.real_frame.texture:SetTexture( LSM:Fetch( "statusbar", self.db.barTexture ) )
    end
end

function RuneBars:GetBarBorder(info)
    return self.db.barBorder
end

function RuneBars:SetBarBorder(info, value)
    self.db.barBorder = value
    self:UpdateBarBorders()
end

function RuneBars:UpdateBarBorders()
    for frame_id, frame in pairs( self.runeFrames ) do
        self:SetBorderForFrame( frame )
    end
end

function RuneBars:GetBorderSize(info)
    return self.db.borderSize
end

function RuneBars:SetBorderSize(info, value)
    self.db.borderSize = value
    self:UpdateBarBorders()
end

function RuneBars:GetBarEdgeOffset(info)
    return self.db.barEdgeOffset
end

function RuneBars:SetBarEdgeOffset(info, value)
    self.db.barEdgeOffset = value
    self:UpdateBarBorders()
end

function RuneBars:IsBarVertical(info)
    return self.db.barVertical
end

function RuneBars:SetBarVertical(info, value)
    self.db.barVertical = value
    self:UpdateBarOrientations()
end

function RuneBars:UpdateBarOrientations()
    for frame_id, frame in pairs( self.runeFrames ) do
        self:UpdateBar( frame.real_frame )
    end
end

function RuneBars:UpdateWidgetsLocking()
    if DKM.db.profile.general.lockWidgets and not self:IsEnabled() then
        for frame_id, frame in pairs( self.runeFrames ) do
            frame:Hide()
        end
    end
end

function RuneBars:UpdateAllRuneTimerVisibility()
    for frame_id, frame in pairs( self.runeFrames ) do
        if self.db.showText then
            frame.real_frame.text:Show()
        else
            frame.real_frame.text:Hide()
        end
    end
end
