--[[
    Title: DKM, Runes module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBaseRunes, L["Unable to locate ModuleBaseRunes object."])

local Runes = DKM:NewModule("Runes", ModuleBaseRunes, "AceConsole-3.0", "AceEvent-3.0")

local RuneTextureNames = {
    [1] = "\\UI-PlayerFrame-Deathknight-Blood",
    [2] = "\\UI-PlayerFrame-Deathknight-Unholy",
    [3] = "\\UI-PlayerFrame-Deathknight-Frost",
    [4] = "\\UI-PlayerFrame-Deathknight-Death"
}

local RuneBorderTextureName = "\\UI-PlayerFrame-Deathknight-Ring"

local RuneColors = {
    [1] = { 1, 0, 0, 1 },
    [2] = { 1, 1, 0, 1 },
    [3] = { 0.5, 0.5, 1, 1 },
    [4] = { 0.7, 0.7, 0.7, 1 }
}

Runes.runeFrames = {}
Runes.runeIDsToFrameIDs = {}

function Runes:OnInitialize()
    DKM.options.args.runes = {
        name = "General",
        type = "group",
        desc = "General options",
        args = {
            hideBlizzardRuneFrame = {
                type = "toggle",
                width = "full",
                name = L["Hide Blizzard Rune Frame"],
                desc = L["Hides Blizzard Rune Frame."],
                handler = Runes,
                get = "IsHideBlizzardRuneFrame",
                set = "ToggleHideBlizzardRuneFrame",
            },
            texture_pack = {
                name = L["Texture pack to use"],
                desc = L["One of texture packs located in 'art/runes/textures' directory."],
                type = "select",
                style = "dropdown",
                handler = Runes,
                get = "GetTexturePack",
                set = "SetTexturePack",
                values = DKM.gfxPacksRunesTextures,
				order = 10
            },
			border_pack = {
                name = L["Border to use"],
                desc = L["One of borders located in 'art/runes/borders' directory."],
                type = "select",
                style = "dropdown",
                handler = Runes,
                get = "GetBorderPack",
                set = "SetBorderPack",
                values = DKM.gfxPacksRunesBorders,
				order = 11
            },
            uiScale = {
                type = "range",
                name = L["Runes UI Scale"],
                desc = L["Scales the runes UI to a factor of the default."],
                min = 0.1,
                max = 2,
                step = 0.05,
                handler = Runes,
                get = "GetUIScale",
                set = "SetUIScale",
                order = 12
            },
            runeCooldownBrightness = {
                type = "range",
                name = L["Brightness on cooldown"],
                desc = L["Selects the brightness of rune's background when it is on cooldown. Make it darker for cooldown text to become more readable."],
                min = 0,
                max = 1,
                step = 0.1,
                handler = Runes,
                get = "GetRuneCooldownBrightness",
                set = "SetRuneCooldownBrightness",
            },
            showCooldown = {
                type = "toggle",
                width = "full",
                name = L["Show animated timer cooldown"],
                desc = L["Shows animated cooldown on rune widget when timer is active."],
                handler = Runes,
                get = "IsShowCooldown",
                set = "SetShowCooldown",
            },
            showText = {
                type = "toggle",
                width = "full",
                name = L["Show timer text"],
                desc = L["Shows text on rune widget when timer is active."],
                handler = Runes,
                get = "IsShowText",
                set = "SetShowText",
            },
            runePriority = {
                type = "toggle",
                width = "full",
                name = L["Rune prioritisation (RECOMMENDED)"],
                desc = L["When enabled, makes rune that is ready or has less cooldown to appear as your 'first' rune. This feature is recommended for Cataclysm."],
                handler = Runes,
                get = "IsRunePriority",
                set = "SetRunePriority",
            }
        }
    }
    
    self.db = DKM.db.profile.runes
    self.options = DKM.options.args.runes
    self:InitBaseOptions(L["Enables "..self:GetName().." module."])
    DKM.optionsFrameRunes = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "Runes", "DKM", "runes")
    
    self:InitializeRunes()
    
    self:SetEnabledState(false)
end

function Runes:OnEnable()
    self:UpdateBlizzardRuneFrame()
    
    self:RegisterEvent("RUNE_TYPE_UPDATE")
    self:RegisterEvent("RUNE_POWER_UPDATE")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_ALIVE")
    
    for frame_id, frame in pairs( self.runeFrames ) do
        self:UpdateRuneCooldownByFrame( frame.real_frame )
        self:UpdateRuneTypeByFrame( frame.real_frame )
        frame:Show()
    end
    
    self:UpdateAllRuneTextures()
    self:UpdateAllRuneTimerVisibility()
    
    DKM:Print(L[self:GetName().." module enabled."])
end

function Runes:OnDisable()
    self:UnregisterEvent("RUNE_TYPE_UPDATE")
    self:UnregisterEvent("RUNE_POWER_UPDATE")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:UnregisterEvent("PLAYER_ALIVE")
    
    for frame_id, frame in pairs( self.runeFrames ) do
        frame:Hide()
    end
    
    DKM:Print(L[self:GetName().." module disabled."])
end

function Runes:CreateRuneFrame(rune_id)
    local frame_base = CreateFrame( "FRAME", "RuneFrameBase"..rune_id, UIParent )
    frame_base:SetPoint( "CENTER", UIParent, DKM.db.profile.runes[rune_id].relativePoint, DKM.db.profile.runes[rune_id].coord_x, DKM.db.profile.runes[rune_id].coord_y )
    frame_base:SetWidth(32)
    frame_base:SetHeight(32)
    frame_base:SetFrameLevel(1)
    frame_base.coords = DKM.db.profile.runes[rune_id]
    frame_base:Hide()

    local frame = CreateFrame( "FRAME", "RuneFrame"..rune_id, frame_base )
    frame:SetAllPoints(frame_base)
    frame:SetFrameLevel(1)
    frame_base.real_frame = frame

    local texture = frame:CreateTexture( "RuneTexture"..rune_id, "ARTWORK" )
    texture:SetAllPoints(frame)
    frame.texture = texture

    local cooldown = CreateFrame( "COOLDOWN", "RuneCooldown"..rune_id, frame )
    cooldown:SetPoint( "CENTER", texture, "CENTER" )
    cooldown:SetWidth(22)
    cooldown:SetHeight(22)
    cooldown:SetFrameLevel(2)
    -- OmniCC Fix
    cooldown.noCooldownCount = true
    frame.cooldown = cooldown

    local border = CreateFrame( "FRAME", "RuneFrameBorder"..rune_id, frame )
    border:SetAllPoints(frame)
    border:SetFrameLevel(3)
    frame.border = border

    local border_texture = border:CreateTexture( "RuneBorderTexture"..rune_id, "OVERLAY" )
    border_texture:SetAllPoints( border )
    border.texture = border_texture

    local text = border:CreateFontString( "RuneFrameText"..rune_id, "OVERLAY", "DKM_UIFont18" )
    text:SetAllPoints(border)
    text:SetTextColor( 1, 1, 0, 1 )
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetText("")
    frame.text = text

    frame.rune_id = rune_id
    frame.cd = 0
    DKM:RegisterDragableFrame( frame_base, rune_id )
    return frame_base
end

function Runes:UpdateRuneColor( frame, ready )
    local runeType = GetRuneType( frame.rune_id )
    
    frame.text:SetTextColor( unpack( RuneColors[ runeType ] ) )
    frame.texture:SetTexture( DKM.gfxPackRunesTexturesData[ self.db.texture_pack ].path..RuneTextureNames[ runeType ] )
end

function Runes:UpdateTextureBrightness( frame, ready )
    if ready then
        frame.texture:SetVertexColor( 1, 1, 1 )
    else
        frame.texture:SetVertexColor( self:GetRuneCooldownBrightness(), self:GetRuneCooldownBrightness(), self:GetRuneCooldownBrightness() )
    end
end

function Runes:UpdateRuneTypeByFrame( frame )
    local start, duration, runeReady = GetRuneCooldown( frame.rune_id )
    
    self:UpdateRuneColor( frame, runeReady )
end

function Runes:UpdateRuneCooldownByFrame( frame )
    local start, duration, runeReady = GetRuneCooldown( frame.rune_id )
    local new_cd
    local old_cd = frame.cd
    
    if not runeReady then
        new_cd = ceil( self:GetRuneTimeLeft( start, duration ) )
        if new_cd < 0 then
            new_cd = 0
        end
    else
        new_cd = 0
    end
    
    if new_cd == frame.cd then
        return
    end
    
    frame.cd = new_cd
    
    if new_cd > 0 then
        if self:IsShowText() then
            frame.text:SetText( frame.cd )
        end
        
        if new_cd > old_cd then
            self:UpdateTextureBrightness( frame, false )
            frame:SetScript( "OnUpdate", function(self, elapsed) Runes:UpdateRuneCooldownByFrame(self) end )
            
            if self.db.showCooldown then
                frame.cooldown:SetCooldown( start, duration )
            end
        end
    else
        frame.cooldown:SetCooldown( 0, 0 )
        frame.text:SetText( "" )
        frame:SetScript( "OnUpdate", nil )
        self:UpdateTextureBrightness( frame, true )
    end
    
    self:UpdateRunePriority( frame.rune_id )
end

function Runes:UpdateRunePriority( rune_id )
    if not self:IsRunePriority() then
        return
    end

    if rune_id % 2 == 0 then
        rune_id = rune_id - 1
    end

    -- If this is removed, it errors out. Dunno why, gotta debug it. Leave it for now as it is.
    if self.runeFrames[rune_id + 1] == nil then
        return
    end

    if self.runeFrames[rune_id].real_frame.cd <= self.runeFrames[rune_id + 1].real_frame.cd then
        return
    end

    self:SwapFrames( rune_id )
end

function Runes:GetTexturePack(info)
    return self.db.texture_pack
end

function Runes:SetTexturePack(info, value)
    self.db.texture_pack = value
    self:UpdateAllRuneTextures()
end

function Runes:GetBorderPack(info)
    return self.db.border_pack
end

function Runes:SetBorderPack(info, value)
    self.db.border_pack = value
    self:UpdateAllRuneTextures()
end

function Runes:UpdateAllRuneTextures()
    for rune_id = 1,#self.runeFrames do
        local runeType = GetRuneType( rune_id )
        self.runeFrames[rune_id].real_frame.texture:SetTexture( DKM.gfxPackRunesTexturesData[ self.db.texture_pack ].path..RuneTextureNames[runeType] )
        self.runeFrames[rune_id].real_frame.border.texture:SetTexture( DKM.gfxPackRunesBordersData[ self.db.border_pack ].path..RuneBorderTextureName )
    end
end

function Runes:GetUIScale(info)
    return self.db.uiScale
end

function Runes:SetUIScale(info, value)
    self.db.uiScale = value
    self:UpdateUIScale()
end

function Runes:UpdateUIScale()
    for rune_id = 1,#self.runeFrames do
        self.runeFrames[rune_id]:SetScale( DKM.db.profile.general.uiScale * self.db.uiScale )
    end
end

function Runes:GetRuneCooldownBrightness(info)
    return self.db.runeCooldownBrightness
end

function Runes:SetRuneCooldownBrightness(info, value)
    self.db.runeCooldownBrightness = value
end

function Runes:IsShowCooldown(info)
    return self.db.showCooldown
end

function Runes:SetShowCooldown(info, value)
    self.db.showCooldown = value
    self:UpdateAllRuneTimerVisibility()
end

function Runes:UpdateAllRuneTimerVisibility()
    for rune_id = 1,#self.runeFrames do
        if self.db.showCooldown then
            self.runeFrames[rune_id].real_frame.cooldown:Show()
        else
            self.runeFrames[rune_id].real_frame.cooldown:Hide()
        end

        if self.db.showText then
            self.runeFrames[rune_id].real_frame.text:Show()
        else
            self.runeFrames[rune_id].real_frame.text:Hide()
        end
    end
end

function Runes:IsRunePriority(info)
    return self.db.runePriority
end

function Runes:SetRunePriority(info, value)
    self.db.runePriority = value
    
    for rune_id = 1, #self.runeFrames, 2 do
        if value then
            self:UpdateRunePriority(1)
        elseif self.runeIDsToFrameIDs[rune_id] ~= rune_id then
            self:SwapFrames( rune_id )
        end
    end
end
