--[[
    Title: DKM, Runic Power module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBase, L["Unable to locate ModuleBase object."])

local RunicPower = DKM:NewModule("RunicPower", ModuleBase, "AceConsole-3.0", "AceEvent-3.0")

function RunicPower:OnInitialize()
    DKM.options.args.runicpower = {
        name = "General",
        type = "group",
        desc = "General options",
        args = {
            uiScale = {
                type = "range",
                name = L["Runic power UI Scale"],
                desc = L["Scales the runic power UI to a factor of the default."],
                min = 0.1,
                max = 2,
                step = 0.05,
                handler = RunicPower,
                get = "GetUIScale",
                set = "SetUIScale",
                order = -1
            }
        }
    }
    
    self.db = DKM.db.profile.runicpower
    self.options = DKM.options.args.runicpower
    self:InitBaseOptions(L["Enables "..self:GetName().." module."])
    DKM.optionsFrameRunicPower = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "Runic Power", "DKM", "runicpower")
    
    self:InitializeRunicPower()
    
    self:SetEnabledState(false)
end

function RunicPower:OnEnable()
    self:RegisterEvent("UNIT_POWER")
    self.frame:SetScript("OnUpdate", function(self, elapsed) RunicPower:OnUpdate(self, elapsed) end)
    self.frame:Show()
    
    DKM:Print(L[self:GetName().." module enabled."])
end

function RunicPower:OnDisable()
    self:UnregisterEvent("UNIT_POWER")
    self.frame:SetScript("OnUpdate", nil)
    self.frame:Hide()
    
    DKM:Print(L[self:GetName().." module disabled."])
end

function RunicPower:InitializeRunicPower()
    frame = CreateFrame("FRAME", "RunicPowerFrame", UIParent)
    frame:SetPoint("CENTER", UIParent, self.db.relativePoint, self.db.coord_x, self.db.coord_y)
    frame:SetWidth(80)
    frame:SetHeight(30)
    frame.coords = DKM.db.profile.runicpower
    frame:Hide()

    local text = frame:CreateFontString("RUNIC_POWER_TEXT", "ARTWORK", "SystemFont_OutlineThick_Huge4")
    text:SetAllPoints(frame)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetTextColor( 1, 0.5, 0, 1)
    text:SetText("0")
    frame.runicPower = text
    frame.lastupdate = 0
    self.frame = frame
    
    DKM:RegisterDragableFrame(frame, "Runic Power")
end

function RunicPower:UpdateRunicPower()
    self.frame.runicPower:SetText( UnitMana( "player" ) )
end

function RunicPower:OnUpdate(frame, elapsed)
    frame.lastupdate = frame.lastupdate + elapsed

    if frame.lastupdate > 0.5 then
        RunicPower:UpdateRunicPower()
        frame.lastupdate = 0
    end
end

function RunicPower:UNIT_POWER(event, ...)
    local who, resource = ...

    if who == "player" then
        self:UpdateRunicPower()
    end
end

function RunicPower:GetUIScale(info)
    return self.db.uiScale
end

function RunicPower:SetUIScale(info, value)
    self.db.uiScale = value
    self:UpdateUIScale()
end

function RunicPower:UpdateUIScale()
    self.frame:SetScale( DKM.db.profile.general.uiScale * self.db.uiScale )
end
