--[[
    Title: DKM, Trackers module
    Author: Sidh@Tarren Mill
    License: GPL
]]--

if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

local L = LibStub("AceLocale-3.0"):GetLocale("DKM")

assert(DKM, L["Unable to locate DK Addon Object."])
assert(ModuleBase, L["Unable to locate ModuleBase object."])

local Trackers = DKM:NewModule("Trackers", ModuleBase, "AceConsole-3.0", "AceEvent-3.0")

local TrackerBorderTextureName = "\\UI-Tooltip-Border"

local TRACKER_DISEASE_BP = 1
local TRACKER_DISEASE_FF = 2
local TRACKER_DISEASE_EP = 3
local TRACKER_DISEASE_SF = 4
local TRACKER_SHADOW_INFUSION = 5
local TRACKER_DARK_TRANSFORMATION = 6
local TRACKER_BLADE_BARRIER = 7
local TRACKER_NECROTIC_STRIKE = 8
local TRACKER_SUDDEN_DOOM = 9
local TRACKER_KILLING_MACHINE = 10
local TRACKER_RIME = 11
local TRACKER_UNHOLY_BLIGHT = 12
local TRACKER_CRIMSON_SCOURGE = 13

local TrackersData = {
    [TRACKER_DISEASE_BP] = {
        spell_id = 55078,
        spell_name = "",
        short_name = "BP",
        texture_name = "\\Spell_DeathKnight_BloodPlague",
    },
    [TRACKER_DISEASE_FF] = {
        spell_id = 55095,
        spell_name = "",
        short_name = "FF",
        texture_name = "\\Spell_DeathKnight_FrostFever",
    },
    [TRACKER_DISEASE_EP] = {
        spell_id = 65142,
        spell_name = "",
        short_name = "EP",
        texture_name = "\\Ability_Creature_Cursed_03",
    },
    [TRACKER_DISEASE_SF] = {
        spell_id = 81130,
        spell_name = "",
        short_name = "SF",
        texture_name = "\\Ability_Creature_Disease_03",
    },
	[TRACKER_SHADOW_INFUSION] = {
		spell_id = 91342,
		spell_name = "",
        short_name = "SI",
        texture_name = "\\Spell_Shadow_Requiem",
    },
	[TRACKER_DARK_TRANSFORMATION] = {
		spell_id = 63560,
		spell_name = "",
        short_name = "DT",
        texture_name = "\\Achievement_Boss_FestergutRotface",
    },
	[TRACKER_BLADE_BARRIER] = {
		spell_id = 49501,
		spell_name = "",
        short_name = "BB",
        texture_name = "\\Ability_UpgradeMoonGlaive",
    },
	[TRACKER_NECROTIC_STRIKE] = {
		spell_id = 73975,
		spell_name = "",
        short_name = "NS",
        texture_name = "\\INV_Axe_96",
    },
	[TRACKER_SUDDEN_DOOM] = {
		spell_id = 81340,
		spell_name = "",
        short_name = "SD",
        texture_name = "\\Spell_Shadow_Painspike",
    },
	[TRACKER_KILLING_MACHINE] = {
		spell_id = 51124,
		spell_name = "",
        short_name = "KM",
        texture_name = "\\INV_Sword_122",
    },
	[TRACKER_RIME] = {
		spell_id = 59052,
		spell_name = "",
        short_name = "RM",
        texture_name = "\\Spell_Frost_ArcticWinds",
    },
	[TRACKER_UNHOLY_BLIGHT] = {
		spell_id = 49194,
		spell_name = "",
        short_name = "UB",
        texture_name = "\\Spell_Shadow_Contagion",
    },
	[TRACKER_CRIMSON_SCOURGE] = {
		spell_id = 81141,
		spell_name = "",
        short_name = "CS",
        texture_name = "\\Ability_Warrior_BloodNova",
    }
}

Trackers.trackerFrames = { }

Trackers.toggleableOptions = { }

function Trackers:OnInitialize()
    DKM.options.args.trackers = {
        name = "General",
        type = "group",
        desc = "General options",
        args = {
            texture_pack = {
                name = L["Texture pack to use"],
                desc = L["One of texture packs located in 'art/trackers/textures' directory."],
                type = "select",
                style = "dropdown",
                handler = Trackers,
                get = "GetTexturePack",
                set = "SetTexturePack",
                values = DKM.gfxPacksTrackersTextures,
                order = 10
            },
			border_pack = {
                name = L["Border to use"],
                desc = L["One of borders located in 'art/trackers/borders' directory."],
                type = "select",
                style = "dropdown",
                handler = Trackers,
                get = "GetBorderPack",
                set = "SetBorderPack",
                values = DKM.gfxPacksTrackersBorders,
                order = 11
            },
            uiScale = {
                type = "range",
                name = L["Trackers UI Scale"],
                desc = L["Scales the trackers UI to a factor of the default."],
                min = 0.1,
                max = 2,
                step = 0.05,
                handler = Trackers,
                get = "GetUIScale",
                set = "SetUIScale",
                order = 12
            },
            showCooldown = {
                type = "toggle",
                width = "full",
                name = L["Show animated timer cooldown"],
                desc = L["Shows animated cooldown on tracker widget when timer is active."],
                handler = Trackers,
                get = "IsShowCooldown",
                set = "SetShowCooldown",
				order = 13
            },
            showText = {
                type = "toggle",
                width = "full",
                name = L["Show timer text"],
                desc = L["Shows text on tracker widget when timer is active."],
                handler = Trackers,
                get = "IsShowText",
                set = "SetShowText",
				order = 14
            },
            subSecondPrescision = {
                type = "toggle",
                width = "full",
                name = L["Sub-second timer prescision"],
                desc = L["Allows tracker timers to show value with sub-second prescision."],
                handler = Trackers,
                get = "GetSubSecondPrescision",
                set = "SetSubSecondPrescision",
                order = 20
            },
            subSecondPrescisionThreshold = {
                type = "range",
                name = L["Sub-second timer prescision threshold"],
                desc = L["Time left on timer when sub-second prescision activates."],
                min = 0.5,
                max = 10,
                step = 0.1,
                handler = Trackers,
                get = "GetSubSecondPrescisionThreshold",
                set = "SetSubSecondPrescisionThreshold",
                order = 21
            },
            fallOffPulse = {
                type = "range",
                name = "Pulse on fall off.",
                desc = "Time left on timer when tracker will pulse.",
                min = 0,
                max = 10,
                step = 0.1,
                handler = Trackers,
                get = "GetFallOffPulse",
                set = "SetFallOffPulse",
                order = 22
            },
			trackers_header = {
				type = "header",
				name = "Trackers",
				order = 30
			},
            showEbonPlague = {
                type = "toggle",
                width = "full",
                name = L["Show Ebon Plague"],
                desc = L["Shows Ebon Plague disease on target."],
                handler = Trackers,
                get = "IsShowEbonPlague",
                set = "SetShowEbonPlague",
            },
            showScarletFever = {
                type = "toggle",
                width = "full",
                name = L["Show Scarlet Fever"],
                desc = L["Shows Scarlet Fever on target."],
                handler = Trackers,
                get = "IsShowScarletFever",
                set = "SetShowScarletFever",
            },
			showUnholyBlight = {
                type = "toggle",
                width = "full",
                name = L["Show Unholy Blight"],
                desc = L["Shows Unholy Blight on target."],
                handler = Trackers,
                get = "IsShowUnholyBlight",
                set = "SetShowUnholyBlight",
            },
			showShadowInfusion = {
                type = "toggle",
                width = "full",
                name = "Show Shadow Infusion",
                desc = "Shows Shadow Infusion buff on ghoul.",
                handler = Trackers,
                get = "IsShowShadowInfusion",
                set = "SetShowShadowInfusion",
            },
			showDarkTransformation = {
                type = "toggle",
                width = "full",
                name = "Show Dark Transformation",
                desc = "Shows Dark Transformation buff on ghoul.",
                handler = Trackers,
                get = "IsShowDarkTransformation",
                set = "SetShowDarkTransformation",
            },
			showBladeBarrier = {
                type = "toggle",
                width = "full",
                name = "Show Blade Barrier",
                desc = "Shows Blade Barrier buff.",
                handler = Trackers,
                get = "IsShowBladeBarrier",
                set = "SetShowBladeBarrier",
            },
			showNecroticStrike = {
                type = "toggle",
                width = "full",
                name = "Show Necrotic Strike",
                desc = "Shows Necrotic Strike debuff on target.",
                handler = Trackers,
                get = "IsShowNecroticStrike",
                set = "SetShowNecroticStrike",
            },
			showSuddenDoom = {
                type = "toggle",
                width = "full",
                name = "Show Sudden Doom",
                desc = "Shows Sudden Doom proc.",
                handler = Trackers,
                get = "IsShowSuddenDoom",
                set = "SetShowSuddenDoom",
            },
			showKillingMachine = {
                type = "toggle",
                width = "full",
                name = "Show Killing Machine",
                desc = "Shows Killing Machine proc.",
                handler = Trackers,
                get = "IsShowKillingMachine",
                set = "SetShowKillingMachine",
            },
			showRime = {
                type = "toggle",
                width = "full",
                name = "Show Rime",
                desc = "Shows Rime proc.",
                handler = Trackers,
                get = "IsShowRime",
                set = "SetShowRime",
            },
			showCrimsonScourge = {
                type = "toggle",
                width = "full",
                name = "Show Crimson Scourge",
                desc = "Shows Crimson Scourge proc.",
                handler = Trackers,
                get = "IsShowCrimsonScourge",
                set = "SetShowCrimsonScourge",
            }
        }
    }
    
    self.db = DKM.db.profile.trackers
    self.options = DKM.options.args.trackers
    self:InitBaseOptions(L["Enables "..self:GetName().." module."])
    DKM.optionsFrameTrackers = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DKM", "Trackers", "DKM", "trackers")
    
    for tracker_id = 1,#TrackersData do
        TrackersData[tracker_id].spell_name = GetSpellInfo(TrackersData[tracker_id].spell_id)
        self:CreateTrackerFrame(tracker_id)
    end
    
    self:SetEnabledState(false)
end

function Trackers:OnEnable()
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_ALIVE")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    self:UpdateAllTrackerTextures()
    self:UpdateTrackers()

    DKM:Print(L[self:GetName().." module enabled."])
end

function Trackers:OnDisable()
    self:UnregisterEvent("PLAYER_TARGET_CHANGED")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:UnregisterEvent("PLAYER_ALIVE")
    self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    for tracker_id = 1,#TrackersData do
        self:UpdateMissingTracker(tracker_id)
        self.trackerFrames[tracker_id]:Hide()
    end

    DKM:Print(L[self:GetName().." module disabled."])
end

function Trackers:CreateTrackerFrame(tracker_id)
    frame = CreateFrame("FRAME", "TrackerFrame"..tracker_id, UIParent)
    frame:SetPoint("CENTER", UIParent, DKM.db.profile.trackers.trackable[tracker_id].relativePoint, DKM.db.profile.trackers.trackable[tracker_id].coord_x, DKM.db.profile.trackers.trackable[tracker_id].coord_y)
    frame:SetWidth(36)
    frame:SetHeight(36)
    frame:SetFrameLevel(1)
    frame.coords = DKM.db.profile.trackers.trackable[tracker_id]

    texture = frame:CreateTexture("TrackerTexture"..tracker_id, "ARTWORK")
    texture:SetAllPoints(frame)
    texture:SetVertexColor(0.8,0.8,0.8)
    frame.texture = texture

    local cooldown = CreateFrame("COOLDOWN", "TrackerCooldown", frame)
    cooldown:SetPoint("CENTER", texture, "CENTER")
    cooldown:SetWidth(25)
    cooldown:SetHeight(25)
    cooldown:SetFrameLevel(3)
    -- OmniCC Fix
    cooldown.noCooldownCount = true
    frame.cooldown = cooldown

    local border = CreateFrame("FRAME", "TrackerFrameBorder", frame)
    border:SetAllPoints(frame)
    border:SetFrameLevel(4)
    frame.border = border

    local border_texture = border:CreateTexture("TrackerBorderTexture"..tracker_id, "OVERLAY")
    border_texture:SetAllPoints(border)
    border.texture = border_texture

    local text = border:CreateFontString("TrackerText"..tracker_id, "OVERLAY", "DKM_UIFont16")
    text:SetAllPoints(border)
    text:SetTextColor(1,1,0,0.7)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetText("")
    frame.text = text

	if tracker_id == TRACKER_SHADOW_INFUSION then
		local stacks_text = border:CreateFontString("TrackerText"..tracker_id, "OVERLAY", "DKM_UIFont14")
		stacks_text:SetAllPoints(border)
		stacks_text:SetTextColor(1,1,0,0.7)
		stacks_text:SetJustifyH("RIGHT")
		stacks_text:SetJustifyV("BOTTOM")
		stacks_text:SetText("")
		frame.stacks_text = stacks_text
	end
	
    frame.tracker_id = tracker_id
    frame.expire_time = 0
    frame.time_since_last_update = 0
    frame.active = false
    frame.found = false
    frame.time_since_last_pulse_update = 0
    frame.current_pulse_scale = 1.0
    frame.pulse_direction_up = true
    
    if DKM.db.profile.general.lockWidgets then
        frame:Hide()
    end

    DKM:RegisterDragableFrame( frame, TrackersData[tracker_id].short_name )
    self.trackerFrames[tracker_id] = frame
end

function Trackers:UpdateTrackers()
	-- Check debuffs on target
    for i = 1,40 do
        local nm, rank, icon, count, debuffType, duration, expire, owner, isSteal = UnitAura("TARGET", i, "HARMFUL")
        if owner == "player" then
			-- TODO: optimize this - we can make map of trackables with key being spellname
			self:UpdateTracker(nm, count, expire, duration, TRACKER_DISEASE_BP)
			self:UpdateTracker(nm, count, expire, duration, TRACKER_DISEASE_FF)
			self:UpdateTracker(nm, count, expire, duration, TRACKER_DISEASE_EP)
			self:UpdateTracker(nm, count, expire, duration, TRACKER_DISEASE_SF)
			self:UpdateTracker(nm, count, expire, duration, TRACKER_NECROTIC_STRIKE)
			self:UpdateTracker(nm, count, expire, duration, TRACKER_UNHOLY_BLIGHT)
--            for tracker_id = 1,#TrackersData do
--                self:UpdateTracker(nm, count, expire, duration, tracker_id)
--            end
        end
    end
	
	-- Check buffs on yourself
	for i = 1,40 do
        local nm, rank, icon, count, debuffType, duration, expire, owner, isSteal = UnitAura("PLAYER", i, "PLAYER")
		self:UpdateTracker(nm, count, expire, duration, TRACKER_BLADE_BARRIER)
		self:UpdateTracker(nm, count, expire, duration, TRACKER_SUDDEN_DOOM)
		self:UpdateTracker(nm, count, expire, duration, TRACKER_KILLING_MACHINE)
		self:UpdateTracker(nm, count, expire, duration, TRACKER_RIME)
		self:UpdateTracker(nm, count, expire, duration, TRACKER_CRIMSON_SCOURGE)
    end
	
	-- Check ghoul's buffs
	for i = 1,40 do
		local nm, rank, icon, count, debuffType, duration, expire, owner, isSteal = UnitAura("PET", i, "HELPFUL")
		self:UpdateTracker(nm, count, expire, duration, TRACKER_SHADOW_INFUSION)
		self:UpdateTracker(nm, count, expire, duration, TRACKER_DARK_TRANSFORMATION)
	end
	
    for tracker_id = 1,#TrackersData do
        self:UpdateMissingTracker(tracker_id)
        self.trackerFrames[tracker_id].found = false
    end

    self:UpdateAllTrackerTimerVisibility()
end

function Trackers:UpdateTracker(nm, count, expire, duration, tracker_id)
    if nm ~= TrackersData[tracker_id].spell_name then
        return
    end

	if ( tracker_id == TRACKER_DISEASE_EP and not self:IsShowEbonPlague() )
	or ( tracker_id == TRACKER_DISEASE_SF and not self:IsShowScarletFever() )
	or ( tracker_id == TRACKER_SHADOW_INFUSION and not self:IsShowShadowInfusion() )
	or ( tracker_id == TRACKER_DARK_TRANSFORMATION and not self:IsShowDarkTransformation() )
	or ( tracker_id == TRACKER_BLADE_BARRIER and not self:IsShowBladeBarrier() )
	or ( tracker_id == TRACKER_NECROTIC_STRIKE and not self:IsShowNecroticStrike() )
	or ( tracker_id == TRACKER_SUDDEN_DOOM and not self:IsShowSuddenDoom() )
	or ( tracker_id == TRACKER_KILLING_MACHINE and not self:IsShowKillingMachine() )
	or ( tracker_id == TRACKER_RIME and not self:IsShowRime() )
	or ( tracker_id == TRACKER_CRIMSON_SCOURGE and not self:IsShowCrimsonScourge() )
	or ( tracker_id == TRACKER_UNHOLY_BLIGHT and not self:IsShowUnholyBlight() ) then
		return
	end
	
    self.trackerFrames[tracker_id].expire_time = expire
    self.trackerFrames[tracker_id].text:SetText( self:FormatTime(expire - GetTime()) )
    self.trackerFrames[tracker_id].cooldown:SetCooldown(expire - duration, duration)
    self.trackerFrames[tracker_id].found = true
    self.trackerFrames[tracker_id].active = true
    self.trackerFrames[tracker_id]:Show()
	
	if tracker_id == TRACKER_SHADOW_INFUSION then
		self.trackerFrames[tracker_id].stacks_text:SetText(count)
	end

    if self.trackerFrames[tracker_id]:GetScript("OnUpdate") == nil then
        self.trackerFrames[tracker_id]:SetScript("OnUpdate", function(frame, elapsed) Trackers:TrackerOnUpdate(frame, elapsed) end)
    end
end

function Trackers:UpdateMissingTracker(tracker_id)
    if self.trackerFrames[tracker_id].found == false then
        self.trackerFrames[tracker_id].active = false

        if self.trackerFrames[tracker_id]:GetScript("OnUpdate") ~= nil then
            self.trackerFrames[tracker_id]:SetScript("OnUpdate", nil)
        end

        self.trackerFrames[tracker_id].text:SetText("")
		
		-- Reset pulse
		self.trackerFrames[tracker_id].time_since_last_pulse_update = 0
		self.trackerFrames[tracker_id].current_pulse_scale = 1.0
		self.trackerFrames[tracker_id].pulse_direction_up = true
		self:UpdateTrackerScale( self.trackerFrames[tracker_id] )
		
        if DKM.db.profile.general.lockWidgets then
            self.trackerFrames[tracker_id]:Hide()
        end
    end
end

function Trackers:TrackerOnUpdate(frame, elapsed)
    frame.time_since_last_update = frame.time_since_last_update + elapsed
    
    if self.db.fallOffPulse > 0 then
        if frame.expire_time - GetTime() > self.db.fallOffPulse then
            frame.time_since_last_pulse_update = 0
            frame.current_pulse_scale = 1.0
            frame.pulse_direction_up = true
        else
            frame.time_since_last_pulse_update = frame.time_since_last_pulse_update + elapsed
            
            if frame.time_since_last_pulse_update >= 0.05 then
                frame.time_since_last_pulse_update = 0
                
                if frame.current_pulse_scale >= 1.3 and frame.pulse_direction_up then
                    frame.pulse_direction_up = false
                elseif frame.current_pulse_scale <= 1 and not frame.pulse_direction_up then
                    frame.pulse_direction_up = true
                end
                
                if frame.pulse_direction_up then
                    frame.current_pulse_scale = frame.current_pulse_scale + 0.05
                else
                    frame.current_pulse_scale = frame.current_pulse_scale - 0.05
                end
            end
        end
        
        self:UpdateTrackerScale( frame )
    end
    
    if frame.time_since_last_update >= 0.1 then
        frame.time_since_last_update = 0
        
        frame.text:SetText( self:FormatTime( frame.expire_time - GetTime() ) )
    end
end

function Trackers:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local _, ev, _, src = ...

    if src == UnitGUID("player") then
        if ev == "SPELL_AURA_APPLIED"
        or ev == "SPELL_AURA_REMOVED"
        or ev == "SPELL_AURA_REFRESH" then
            self:UpdateTrackers()
        end
    end
end

function Trackers:PLAYER_ENTERING_WORLD(event, ...)
   self:UpdateTrackers()
end

function Trackers:PLAYER_ALIVE(event, ...)
    self:UpdateTrackers()
end

function Trackers:PLAYER_TARGET_CHANGED(event, ...)
    self:UpdateTrackers()
end

function Trackers:GetTexturePack(info)
    return self.db.texture_pack
end

function Trackers:SetTexturePack(info, value)
    self.db.texture_pack = value
    self:UpdateAllTrackerTextures()
end

function Trackers:GetBorderPack(info)
    return self.db.border_pack
end

function Trackers:SetBorderPack(info, value)
    self.db.border_pack = value
    self:UpdateAllTrackerTextures()
end

function Trackers:UpdateAllTrackerTextures()
    for tracker_id = 1,#TrackersData do
        self.trackerFrames[tracker_id].texture:SetTexture( DKM.gfxPackTrackersTexturesData[ self.db.texture_pack ].path..TrackersData[tracker_id].texture_name )
        self.trackerFrames[tracker_id].border.texture:SetTexture()
        self.trackerFrames[tracker_id].border.texture:SetTexture( DKM.gfxPackTrackersBordersData[ self.db.border_pack ].path..TrackerBorderTextureName )

        self.trackerFrames[tracker_id].cooldown:SetWidth(
            self.trackerFrames[tracker_id]:GetWidth()
            * DKM.textureFormCooldownScaleFactor[ DKM.gfxPackTrackersTexturesData[ self.db.texture_pack ].form ]
        )
        self.trackerFrames[tracker_id].cooldown:SetHeight(
            self.trackerFrames[tracker_id]:GetHeight()
            * DKM.textureFormCooldownScaleFactor[ DKM.gfxPackTrackersTexturesData[ self.db.texture_pack ].form ]
        )
    end
end

function Trackers:GetUIScale(info)
    return self.db.uiScale
end

function Trackers:SetUIScale(info, value)
    self.db.uiScale = value
    self:UpdateUIScale()
end

function Trackers:UpdateUIScale()
    for tracker_id = 1,#self.trackerFrames do
        self:UpdateTrackerScale( self.trackerFrames[tracker_id] )
    end
end

function Trackers:UpdateTrackerScale( frame )
    frame:SetScale( DKM.db.profile.general.uiScale * self.db.uiScale * frame.current_pulse_scale )
    frame:ClearAllPoints()
    frame:SetPoint(
        "CENTER",
        UIParent,
        DKM.db.profile.trackers.trackable[frame.tracker_id].relativePoint,
        DKM.db.profile.trackers.trackable[frame.tracker_id].coord_x / frame.current_pulse_scale,
        DKM.db.profile.trackers.trackable[frame.tracker_id].coord_y / frame.current_pulse_scale
    )
    
    -- This fixes cooldown animation with tracker fall-off pulse (cooldown animation behaves very badly when you scale + reposition)
    if self:GetFallOffPulse() then
        frame:Hide()
        frame:Show()
    end
end

function Trackers:UpdateWidgetsLocking()
    -- Update trackers if we locked widgets - this will make them disapear if needed
    if DKM.db.profile.general.lockWidgets then
        self:UpdateTrackers()
    end
end

function Trackers:GetSubSecondPrescision(info)
    return self.db.subSecondPrescision
end

function Trackers:SetSubSecondPrescision(info, value)
    self.db.subSecondPrescision = value
end

function Trackers:GetSubSecondPrescisionThreshold(info)
    return self.db.subSecondPrescisionThreshold
end

function Trackers:SetSubSecondPrescisionThreshold(info, value)
    self.db.subSecondPrescisionThreshold = value
end

function Trackers:IsShowCooldown(info)
    return self.db.showCooldown
end

function Trackers:SetShowCooldown(info, value)
    self.db.showCooldown = value
    self:UpdateAllTrackerTimerVisibility()
end

function Trackers:IsShowEbonPlague(info)
    return self.db.showEbonPlague
end

function Trackers:SetShowEbonPlague(info, value)
    self.db.showEbonPlague = value
    self:UpdateEbonPlagueVisibility()
end

function Trackers:UpdateEbonPlagueVisibility()
    if self.db.showEbonPlague then
        self.trackerFrames[ TRACKER_DISEASE_EP ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_DISEASE_EP ]:Hide()
    end
end

function Trackers:IsShowScarletFever(info)
    return self.db.showScarletFever
end

function Trackers:SetShowScarletFever(info, value)
    self.db.showScarletFever = value
    self:UpdateScarletFeverVisibility()
end

function Trackers:UpdateScarletFeverVisibility()
    if self.db.showScarletFever then
        self.trackerFrames[ TRACKER_DISEASE_SF ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_DISEASE_SF ]:Hide()
    end
end

function Trackers:IsShowUnholyBlight(info)
    return self.db.showUnholyBlight
end

function Trackers:SetShowUnholyBlight(info, value)
    self.db.showUnholyBlight = value
    self:UpdateUnholyBlightVisibility()
end

function Trackers:UpdateUnholyBlightVisibility()
    if self.db.showUnholyBlight then
        self.trackerFrames[ TRACKER_UNHOLY_BLIGHT ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_UNHOLY_BLIGHT ]:Hide()
    end
end

function Trackers:IsShowShadowInfusion(info)
    return self.db.showShadowInfusion
end

function Trackers:SetShowShadowInfusion(info, value)
    self.db.showShadowInfusion = value
    self:UpdateShadowInfusionVisibility()
end

function Trackers:UpdateShadowInfusionVisibility()
    if self.db.showShadowInfusion then
        self.trackerFrames[ TRACKER_SHADOW_INFUSION ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_SHADOW_INFUSION ]:Hide()
    end
end

function Trackers:IsShowDarkTransformation(info)
    return self.db.showDarkTransformation
end

function Trackers:SetShowDarkTransformation(info, value)
    self.db.showDarkTransformation = value
    self:UpdateDarkTransformationVisibility()
end

function Trackers:UpdateDarkTransformationVisibility()
    if self.db.showDarkTransformation then
        self.trackerFrames[ TRACKER_DARK_TRANSFORMATION ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_DARK_TRANSFORMATION ]:Hide()
    end
end

function Trackers:IsShowBladeBarrier(info)
    return self.db.showBladeBarrier
end

function Trackers:SetShowBladeBarrier(info, value)
    self.db.showBladeBarrier = value
    self:UpdateBladeBarrierVisibility()
end

function Trackers:UpdateBladeBarrierVisibility()
    if self.db.showBladeBarrier then
        self.trackerFrames[ TRACKER_BLADE_BARRIER ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_BLADE_BARRIER ]:Hide()
    end
end

function Trackers:IsShowNecroticStrike(info)
    return self.db.showNecroticStrike
end

function Trackers:SetShowNecroticStrike(info, value)
    self.db.showNecroticStrike = value
    self:UpdateNecroticStrikeVisibility()
end

function Trackers:UpdateNecroticStrikeVisibility()
    if self.db.showNecroticStrike then
        self.trackerFrames[ TRACKER_NECROTIC_STRIKE ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_NECROTIC_STRIKE ]:Hide()
    end
end

function Trackers:IsShowSuddenDoom(info)
    return self.db.showSuddenDoom
end

function Trackers:SetShowSuddenDoom(info, value)
    self.db.showSuddenDoom = value
    self:UpdateSuddenDoomVisibility()
end

function Trackers:UpdateSuddenDoomVisibility()
    if self.db.showSuddenDoom then
        self.trackerFrames[ TRACKER_SUDDEN_DOOM ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_SUDDEN_DOOM ]:Hide()
    end
end

function Trackers:IsShowKillingMachine(info)
    return self.db.showKillingMachine
end

function Trackers:SetShowKillingMachine(info, value)
    self.db.showKillingMachine = value
    self:UpdateKillingMachineVisibility()
end

function Trackers:UpdateKillingMachineVisibility()
    if self.db.showKillingMachine then
        self.trackerFrames[ TRACKER_KILLING_MACHINE ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_KILLING_MACHINE ]:Hide()
    end
end

function Trackers:IsShowRime(info)
    return self.db.showRime
end

function Trackers:SetShowRime(info, value)
    self.db.showRime = value
    self:UpdateRimeVisibility()
end

function Trackers:UpdateRimeVisibility()
    if self.db.showRime then
        self.trackerFrames[ TRACKER_RIME ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_RIME ]:Hide()
    end
end

function Trackers:IsShowCrimsonScourge(info)
    return self.db.showCrimsonScourge
end

function Trackers:SetShowCrimsonScourge(info, value)
    self.db.showCrimsonScourge = value
    self:UpdateCrimsonScourgeVisibility()
end

function Trackers:UpdateCrimsonScourgeVisibility()
    if self.db.showCrimsonScourge then
        self.trackerFrames[ TRACKER_CRIMSON_SCOURGE ]:Show()
        self:UpdateTrackers()
    else
        self.trackerFrames[ TRACKER_CRIMSON_SCOURGE ]:Hide()
    end
end

function Trackers:GetFallOffPulse(info)
    return self.db.fallOffPulse
end

function Trackers:SetFallOffPulse(info, value)
    self.db.fallOffPulse = value
end

function Trackers:IsShowText(info)
    return self.db.showText
end

function Trackers:SetShowText(info, value)
    self.db.showText = value
    self:UpdateAllTrackerTimerVisibility()
end

function Trackers:UpdateAllTrackerTimerVisibility()
    for tracker_id = 1,#TrackersData do
        if self.db.showCooldown then
            self.trackerFrames[ tracker_id ].cooldown:Show()
        else
            self.trackerFrames[ tracker_id ].cooldown:Hide()
        end

        if self.db.showText then
            self.trackerFrames[ tracker_id ].text:Show()
        else
            self.trackerFrames[ tracker_id ].text:Hide()
        end
    end
end

function Trackers:FormatTime( time )
    return DKM:FormatTime( time, self.db.subSecondPrescision, self.db.subSecondPrescisionThreshold )
end
